      program pirum
C***********************************************************************
C
C     PIRUM (PROGRAM FOR INDEXING AND REFINEMENT OF UNIT-CELL MATRIX)
C     DETERMINATION OF ACCURATE CELL PARAMETERS FROM POWDER DATA BY THE
C     METHOD OF THE LEAST SQUARES. MAX 200 SIN SQUARE THETA OBS.
C
C     (WRITTEN BY P-E WERNER,STOCKHOLM,1967)
C     PIRUM VERSION  1980.05.05
C
C     MENUE-CONTROLLED VERSION MADE BY L.ERIKSSON FEBR. 1992
C
C     HESS WEIGTHS, USE OF D1 AS MAX DEVIATIONS IN DEG. 2-THETA
C     (D2 AND SSQTL SKIPPED), FIXED (HKL) REFINEMENTS (cf. PURUM)
C     P.-E. WERNER AUG. 1992
C***********************************************************************
C     SIO2 850125          ! This is the title line.
C     0.03285   2108.00    ! Sine square theta and Intensity. 
C     0.05282   9253.00    ! (intensity can be omitted...)
C     0.09851    877.00    !     .
C     0.11424   1009.00    !     .
C     0.11885    499.00    !     .
C     0.13138    565.00    !     .
C     0.15165    522.00    !     .
C     0.17987   2375.00    !     .
C     0.18320     34.00    !     .
C     0.21270    614.00    !     .
C     0.21599    281.00    !     .
C     0.22966     34.00    !     .
C     0.25015   1564.00    !     .
C     0.28174    334.00    !     .
C     0.29530     46.00    !     .
C     0.31124   1314.00    !     .
C     0.31462   1774.00    !     .
C     0.31588    629.00    !     .
C     0.35854    492.00    !     .
C     0.37713    661.00    !     .
C     0.39413    342.00    !     .
C     0.41326   1017.00    !     .
C     0.42453    778.00    !     .
C     0.42713    620.00    !     .
C     0.44750    514.00    !     .
C     0.45732     62.00    !     .
C     0.47924     81.00    !     .
C                          ! a blank line to indicate end of SSQ-data.
C     0.030000  0.000000  0.000000  1.540598   0   0   0  4  8  0
C     4.913400  5.405200  0.000000  0.000000   0.000000  0.000000  1
C-----------------------------------------------------------------------
C   The last two lines are the control parameters.
C-----------------------------------------------------------------------
C
      character titstr*80, answer*1, command*80, cpar*80, fname*80
      integer   keys,ludisp
      integer   exsymb(50,5),numext,ilatt
      logical   manflg, okflag, empty, dflag, flag

      dimension amat(6,6),bmat(6,1),s(6,6),f(6),iht(400),
     >          ikt(400),ilt(400),iqe(400),t1(200),t2(400),tagut(200),
     >          t11(400),t22(400),wg(400),ihi(400),iki(400),ili(400)
c
c --- Logical unit numbers ...
c


c
      lukeys = 5
      ludisp = 6
      iin    = 7
      iout   = 8
      lulist = 8
      ioup   = 9
      linfo  = 10
c
c --- some defalt values ...
c
      okflag = .false.
      dflag  = .false.
      numext = 0
      ilatt  = 1
      isub   = 0
      inr    = 5
      isym   = 1
      iomv   = 1
      a11    = 10.0
      a22    = 0.0
      a33    = 0.0
      a13    = 0.0
      a12    = 0.0
      a23    = 0.0
      dt     = 0.03
      wt     = 0.0
      d2     = 0.0
      ip     = 0
      wave   = 1.5405981
      ihm    = 10
      ikm    = 10
      ilm    = 10
      n1     = 0
      titstr = ' '
c
c --- write initial welcome text and read command ...
c
      call ersscr(ludisp)
      write (ludisp,'(a)')
     >' PIRUM version 921204. Type HELP to get some information'
      write (ludisp,'(a)')
     >' Default IP=0 (i.e. No (hkl) on input data). '
      write (ludisp,'(a)') ' '
  110 continue
      write (ludisp,'(a,$)') ' Pirum> '
      read  (lukeys,'(a)') command
      if (command .eq. ' ') then
         goto 110
      endif
      call strip(command)
      n = nextbl(command)
      cpar = command(n:80)
      if (cpar .ne. ' ') then
         command(n:80) = ' '
         call strip(cpar)
      endif
      call upline(command)
c
c --- decode the command ...
c
      if     (command .eq. 'CELL' .or.
     >        command .eq. 'CEL' .or.
     >        command .eq. 'CE' ) then
         call chaaij (lukeys,ludisp,iomv,isym,a11,a22,a33,a13,a12,a23)

      elseif (command .eq. 'CHANGE' .or.
     >        command .eq. 'CHANG' .or.
     >        command .eq. 'CHAN' .or.
     >        command .eq. 'CHA' .or.
     >        command .eq. 'CH' ) then
         call pircha (lukeys,ludisp,iin,iout,titstr,
     >                t1,tagut,n1,
     >                dt,wt,ip,wave,ihm,ikm,ilm,isym,inr,isub,
     >                exsymb,numext,ilatt,
     >                a11,a22,a33,a13,a12,a23,iomv)

      elseif (command .eq. 'CONDITION' .or.
     >        command .eq. 'CONDITIO' .or.
     >        command .eq. 'CONDITI' .or.
     >        command .eq. 'CONDIT' .or.
     >        command .eq. 'CONDI' .or.
     >        command .eq. 'COND' ) then
         call exttop (lukeys,ludisp,numext,exsymb,ilatt,isub)

      elseif (command .eq. 'CONVERT' .or.
     >        command .eq. 'CONVER' .or.
     >        command .eq. 'CONVE' .or.
     >        command .eq. 'CONV' ) then
         call myconv (lukeys,ludisp,t1,n1,wave)

      elseif (command .eq. 'CREATE' .or.
     >        command .eq. 'CREAT' .or.
     >        command .eq. 'CREA' .or.
     >        command .eq. 'CRE' .or.
     >        command .eq. 'CR' ) then
         if (okflag) then
            call crefil (lukeys,ludisp,iout,titstr,
     >                   t1,tagut,n1,
     >                   dt,wt,ip,wave,ihm,ikm,ilm,isym,inr,isub,
     >                   exsymb,numext,ilatt,
     >                   a11,a22,a33,a13,a12,a23,iomv)
         else
            write (ludisp,'(a)') ' '
            write (ludisp,'(a)') 
     >      ' You must read some parameters first !'
            write (ludisp,'(a)') ' '
            write (ludisp,'(a)')
     >      ' Use the command PREAD or QREAD to do this ...'
            write (ludisp,'(a)') ' '
         endif

      elseif (command .eq. 'CYCLES' .or.
     >        command .eq. 'CYCLE' .or.
     >        command .eq. 'CYCL' .or.
     >        command .eq. 'CYC' .or.
     >        command .eq. 'CY' ) then
         call chainr(lukeys,ludisp,inr)

      elseif (command .eq. 'DT' ) then
         call chad1(lukeys,ludisp,dt)

      elseif (command .eq. 'IP')  then
         call chad2(lukeys,ludisp,ip)
      elseif (command .eq. 'DISPLAY' .or. 
     >        command .eq. 'DISPLA' .or.
     >        command .eq. 'DISPL' .or.
     >        command .eq. 'DISP' .or.
     >        command .eq. 'DIS' .or.
     >        command .eq. 'DI' ) then
         if (dflag) then
            dflag = .false.
            write (ludisp,'(a)') ' Output of results on a file.'
         else
            dflag = .true.
            write (ludisp,'(a)') ' Output of results on the display.'
         endif

      elseif (command .eq. 'ERASE' .or.
     >        command .eq. 'ERAS' .or.
     >        command .eq. 'ERA' .or.
     >        command .eq. 'ER' ) then
         call ersscr(ludisp)
       elseif (command .eq. 'INFO' .or.
     >         command .eq. 'INF' .or.
     >         command .eq. 'IN') then
          call inform(ludisp,lukeys,linfo)
          call fnhelp (ludisp)
      elseif (command .eq. 'EXIT' .or.
     >        command .eq. 'EXI' .or.
     >        command .eq. 'EX' ) then
         goto 680

      elseif (command .eq. 'HELP' .or.
     >        command .eq. 'HEL' .or.
     >        command .eq. 'HE' ) then
         call fnhelp (ludisp)

      elseif (command .eq. 'HKLMAX' .or.
     >        command .eq. 'HKLMA' .or.
     >        command .eq. 'HKLM' .or.
     >        command .eq. 'HKL' ) then
         call chahklmax(lukeys,ludisp,ihm,ikm,ilm)

      elseif (command .eq. 'LOOK' .or.
     >        command .eq. 'LOO' .or.
     >        command .eq. 'LO' ) THEN
         call pilook (lukeys,ludisp,iin,iout,titstr,
     >                t1,tagut,n1,
     >                dt,wt,ip,wave,ihm,ikm,ilm,isym,inr,isub,
     >                exsymb,numext,ilatt,
     >                a11,a22,a33,a13,a12,a23,iomv)

      elseif (command .EQ. 'PREAD' .OR.
     >        command .eq. 'PREA' .or.
     >        command .eq. 'PRE' .or.
     >        command .eq. 'PR' ) THEN
              okflag = .true.
              call prdnew (lukeys,ludisp,iin,titstr,cpar,
     >                     t1,tagut,n1,
     >                     dt,wt,ip,wave,ihm,ikm,ilm,isym,inr,isub,
     >                     exsymb,numext,ilatt,
     >                     a11,a22,a33,a13,a12,a23,iomv,
     >                     iht,ikt,ilt,ihi,iki,ili)
        if(ip .eq. 2) then
        ip=3
        write(ludisp,'(a)') ' CYCLES and CELL parameters not used '
        write(ludisp,'(a)') ' Set SYSTEM parameter '
        goto 110
        endif
      elseif (command .EQ. 'QREAD' .or.
     >        command .EQ. 'QREA' .or.
     >        command .EQ. 'QRE' .or.
     >        command .EQ. 'QR' ) then
         okflag = .true.
         call getspc (lukeys,ludisp,iin,t1,tagut,n1,titstr)
         write(ludisp,'(a,/,a,/,a)')
     > ' Normal first run: Give SYST, CELL, (ev. COND and COND.INP) ',
     > ' If other input data (not sine square theta) use CONVERT ',
     > ' Use DI (for test run output on display to select DT) ', 
     > ' Use CREATE before you leave PIRUM (next time use PREAD) '
      elseif (command .eq. 'RUN' .or.
     >        command .eq. 'RU' .or.
     >        command .eq. 'R' ) THEN
         if (okflag) then
            if (n1 .gt. 0) then
               lernum = inr
               goto 130
            else
               write (ludisp,'(a)') ' '
               write (ludisp,'(a)')
     >' You do not have any sine square theta and intensity values...'
               write (ludisp,'(a)') ' '
            endif
         else
            write (ludisp,'(a)') ' '
            write (ludisp,'(a)')
     >      ' You must read some parameters first !'
            write (ludisp,'(a)') ' '
            write (ludisp,'(a)')
     >      ' Use command PREAD or QREAD to do this ...'
            write (ludisp,'(a)') ' '
         endif

C      elseif (command .eq. 'SSQTL' .or.
C     >        command .eq. 'SSQT' .or.
C     >        command .eq. 'SSQ' .or.
C     >        command .eq. 'SS' ) then
       elseif (command .eq. 'WT' ) then
         call chassqtl(lukeys,ludisp,wt)

      elseif (command .eq. 'SYSTEM' .or.
     >        command .eq. 'SYSTE' .or.
     >        command .eq. 'SYST' .or.
     >        command .eq. 'SYS' .or.
     >        command .eq. 'SY' ) then
         call chasys (lukeys,ludisp,isym)

      elseif (command .eq. 'TITLE' .or.
     >        command .eq. 'TITL' .or.
     >        command .eq. 'TIT' .or.
     >        command .eq. 'TI' ) then
         call chatit(lukeys,ludisp,titstr)

      elseif (command .eq. 'TYPE' .or.
     >        command .eq. 'TYP' .or.
     >        command .eq. 'TY' ) then
         call tyfile (lukeys,ludisp,iin,cpar)

      elseif (command .eq. 'WAVE' .or.
     >        command .eq. 'WAV' .or.
     >        command .eq. 'WA' ) then
         call chawave(lukeys,ludisp,wave)

      else
         write(ludisp,'(a)') ' Unknown command ...'

      endif
c
c --- Loop back to the writeout of the prompter...
c
      goto 110
c
c***********************************************************************
c --- Ok, everything ready lets run the program...
c***********************************************************************
c
  130 continue
c  
       if(ip .eq. 3) goto 700
      goto 140
  700 continue
c-----------------------------------------------------------------------
c     Open the output file and then run the program PURUM part. ...
c-----------------------------------------------------------------------
      if (dflag) then
         iout = ludisp
         goto 780
      else
         iout = lulist
      endif

  702 write (ludisp,'(a,$)') ' Output file name .... : '
      read (lukeys,'(a)') fname
      inquire(file=fname,exist=flag)
        if(flag) then
        write(ludisp,'(a,a,$)')
     >' This file already exists,',
     >' Do you wish to overwrite it ? (Y/N[Def.]'
        read(lukeys,'(a)') answer
          if(answer .ne. 'y' .and. answer .ne. 'Y') then
          goto 702
          endif
        endif
      if (fname .eq. ' ') then
         write (ludisp,'(a)') ' Blank filenames are not allowed !'
         goto 110
      endif
C      open (unit=iout,file=fname,status='new',err=781)
      open (unit=iout,file=fname,status='unknown',
     >      iostat=istat)
       if(istat .ne. 0) then
       write(ludisp,'(a,a,a)')
     > ' Error opening',fname,' try again..'
       goto 702
       endif
  780 CONTINUE
C 
C    Now use fixed (hkl) to calc. a11.....a23
       if (dflag) then
       iout=ludisp
       endif
      i=0
      do while (i .lt. n1)
      i=i+1
      ha=sqrt(t1(i))
      hb=sqrt(1.-t1(i))
      t11(i)=114.59156*atan(ha/hb)
      ssqto=t1(i)
      ha=sqrt(ssqto)
      hb=sqrt(1.-ssqto)
      t22(i)=114.59156*atan(ha/hb)
       if(wt .eq. 1.) then
         wg(i)=1.
         else 
       wg(i)=1./((2.*ha*hb)**2)
       endif
      iqe(i)=1
      enddo
      call pw58(ihi,iki,ili,n1,isym)
      call p55w(t1,wg,iqe,iht,ikt,ilt,n1,isym,amat,bmat,iekv)
      if(isym .eq. 1) nsy=1
      if(isym .eq. 2 .or. isym .eq. 4) nsy=2
      if(isym .eq. 3) nsy=3
      if(isym .eq. 5) nsy=4
      if(isym .eq. 6) nsy=6
      call pw54(amat,nsy,bmat)
      a11=bmat(1,1)
      a22=bmat(2,1)
      a33=bmat(3,1)
      a13=bmat(4,1)
      a12=bmat(5,1)
      a23=bmat(6,1)
       if (dflag) then
       iout=ludisp
       endif
      write(iout,'(6f10.6)') a11,a22,a33,a13,a12,a23
c     calc. error square sum (v2) and (delta-2heta) square sum.
      call pw71(t1,wg,iht,ikt,ilt,iqe,isym,n1,a11,a22,a33,a13,a12,a23,
     >          v2,v22,t22,nekv)      
c     calc. direct cell parameters.
      call pw63(a11,a22,a33,a13,a12,a23,wave,isym,a,b,c,al,be,ga)
c     calc. partial derivatives of direct cell parameters (in s(6,6))
       if(isym .eq. 6) then
       call pw61(a,b,c,al,be,ga,s)
       else
       call pw62(a,b,c,be,s,isym)
       endif
c     calc. direct cell standard deviations --from amat(6,6), s(6,6)
c     and cell parameters.
      call pw52(nsy,nekv,isym,v2,wave,amat,s,a,b,c,al,be,ga,
     >          da,db,dc,dal,dbe,dga,volm)
c
c
c     calc. ssqt-max and 
      ssqtm=(sin((0.5*t11(n1)+2.0*dt)*0.01745329))**2
      if(n1-30) 710,711,711
  711 nt30=30
      sq30=t1(30)
      go to 715
  710 nt30=0
      sq30=0
      if(n1-20) 716,715,715
  715 nt20=20
      sq20=t1(20)
      go to 717
  716 nt20=0
      sq20=0.0
  717 n2=0
      ih=0
      ik=0
      il=0
      n20=0
      n30=0
      nfi=0
      sqfi=t1(n1)
  733 call pw50
     >     (a11,a22,a33,a13,a12,a23,isym,ihm,ikm,ilm,ih,ik,il,ssqt,ia)
      if(ia) 730,750,730
  730 if(ssqt-ssqtm) 731,731,733
  731 if(isub) 732,734,732
  732 call ex(ih,ik,il,ij,numext,exsymb,ilatt)
      if(ij) 734,733,734
  734 call pw51(ssqt,t1,n1,t11,dt,ssqto,ia)
      if(ssqt-sqfi) 735,735,736
  735 call pw80(ssqt,t2,n2,iht,ikt,ilt,ih,ik,il,isym,inne)
      if(inne) 737,737,738
  737 nfi=nfi+1
  736 if(ssqt-sq30) 739,739,740
  739 n30=n30+1
  740 if(ssqt-sq20) 741,741,738
  741 n20=n20+1
  738 if(ia) 742,733,742
  742 call pw53(ssqt,t2,t22,wg,n2,iht,ikt,ilt,iqe,ih,ik,il,isym,wt)
      if(ssqt) 743,733,733
  743 n2=n2+1
      if(n2-400) 733,750,750
750   continue
         write(iout,751)
     >   ' Number of lines (=number of equations) =',NEKV,
     >   ' A =',A,DA,' A   Alfa  =',AL,DAL,' Degrees',
     >   ' B =',B,DB,' A   Beta  =',BE,DBE,' Degrees',
     >   ' C =',C,DC,' A   Gamma =',GA,DGA,' Degrees',
     >   ' Unit cell volume =',VOLM,' A**3'
 751     format(' ',A,I5,/,3(' ',A,2F10.6,A,2F10.6,A,/),A,F10.2,A)
      call pw57(t1,t2,iht,ikt,ilt,n1,n2,iout,dt,wt,ip,wave,tagut,n20,
     >          n30,nfi,nt20,nt30,sq20,sq30,sqfi,m20,noind,t11,t22,
     >          ihi,iki,ili,ioup,a11,a22,a33,a13,a12,a23)
      iout=-iout
      call pw56(t1,t2,iht,ikt,ilt,n1,n2,iout,dt,wt,ip,wave,tagut,n20,
     >          n30,nfi,nt20,nt30,sq20,sq30,sqfi,m20,noind,t11,t22,ioup)
       do 776 i=1,n1
       iht(i)=ihi(i)
       ikt(i)=iki(i)
       ilt(i)=ili(i)
 776   continue
       goto 777
c      write(iout,'(a,f10.6)')
c     >      ' Summa(Delta(2th)**2)=',v22
c----------------------------------------------------------------------+
c --- For this time end of the calculations, close output file and     !
c     set the number of entries in the sine square theta table to      !
c     the original value (It is decreased to zero by some routines).   !
c     At last return to the main many.                                 !
      stop
  140 continue
c
c
c --- First check that all spacing data are less than 1.0
c
      do 141 i = 1,n1
         if (t1(i) .gt. 1.0) then
            write (ludisp,'(a)') ' '
            write (ludisp,'(a,i3,a,f12.6,a)')
     >      ' WARNING, observed spacing:',i,' = ',t1(i),' which is > 1'
            write (ludisp,'(a)') ' '
            write (ludisp,'(a,$)')
     >      ' Do you want to continue ? <Y/N[Def.]>'
            read (lukeys,'(a)') answer
            if (answer .ne. 'y' .and. answer .ne. 'Y') then
               goto 110
            endif
         endif
      ha=sqrt(t1(i))
      hb=sqrt(1.-t1(i))
      t11(i)=114.59156*atan(ha/hb)
C      write(ludisp,'(a,f10.5,a,f10.5)')
C     >      ' ssq-obs:',t1(i),' two-theta:',t11(i)
 141  continue

c-----------------------------------------------------------------------
c     Open the output file and then run the program PIRUM ...
c-----------------------------------------------------------------------
      if (dflag) then
         iout = ludisp
         goto 160
      else
         iout = lulist
      endif
  150 write (ludisp,'(a,$)') ' List file name .... : '
      read (lukeys,'(a)') fname
      inquire(file=fname,exist=flag)
        if(flag) then
        write(ludisp,'(a,a,$)')
     >' This file already exists,',
     >' Do you wish to overwrite it ? (Y/N[Def.]'
        read(lukeys,'(a)') answer
          if(answer .ne. 'y' .and. answer .ne. 'Y') then
          goto 150
          endif
        endif
      if (fname .eq. ' ') then
         write (ludisp,'(a)') ' Blank filenames are not allowed !'
         goto 150
      endif
      open (unit=iout,file=fname,status='unknown',
     >      iostat=istat)
       if(istat .ne. 0) then
       write(ludisp,'(a,a,a)')
     > ' Error opening',fname,' try again..'
       goto 150
       endif
  160 CONTINUE
C
      if (ip .eq. 1) then
       write (ludisp,'(a,$)') ' Reflexion output file name .... : '
       read (lukeys,'(a)') fname
      inquire(file=fname,exist=flag)
        if(flag) then
        write(ludisp,'(a,a,$)')
     >' This file already exists,',
     >' Do you wish to overwrite it ? (Y/N[Def.]'
        read(lukeys,'(a)') answer
          if(answer .ne. 'y' .and. answer .ne. 'Y') then
          goto 160
          endif
        endif
      if (fname .eq. ' ') then
         write (ludisp,'(a)') ' Blank filenames are not allowed !'
         goto 160
      endif
      open (unit=ioup,file=fname,status='unknown',
     >      iostat=istat)
       if(istat .ne. 0) then
       write(ludisp,'(a,a,a)')
     > ' Error opening',fname,' try again..'
       goto 160
       endif
      endif
C
      CONTINUE
c
c --- If IOMV <> 0 then convert the real cell parameters
c     to normal linear parameters, and set IOMV = 0
c
      if (iomv .ne. 0) then
         call rdcel(a11,a22,a33,a13,a12,a23,isym,wave)
         iomv = 0
      endif
c
c --- Write the title to the output file.
c
      ler1 = istart(titstr)
      ler2 = last(titstr)
      if (ler2 .gt. ler1) then
         write(iout,'(a,a)') ' ',titstr(ler1:ler2)
      else
         write(iout,'(a)') ' Input data.'
      endif

      write(iout,'(6f10.6,i10)') a11,a22,a33,a13,a12,a23,iomv
      write(iout,'(a,i5)') ' Number of observed lines = ',N1
      write(iout,'(a)') ' Cycle results.'
      if(ihm+ikm+ilm) 280,190,280
C  190 ssqtm=t1(n1)+2*d2
  190 ssqtm=(sin((0.5*t11(n1)+2.0*dt)*0.01745329))**2
  200 ihm=ihm+1
      if(ssqtm-ihm*ihm*a11) 210,200,200
  210 ihm=ihm-1
      go to(220,230,250,230,250,250),isym
  220 ikm=ihm
      ilm=ihm
      go to 280
  230 ikm=ihm
  240 ilm=ilm+1
      if(ssqtm-ilm*ilm*a22) 280,240,240
  250 ikm=ikm+1
      if(ssqtm-ikm*ikm*a22) 260,250,250
  260 ikm=ikm-1
  270 ilm=ilm+1
      if(ssqtm-ilm*ilm*a33) 280,270,270
  280 iv=inr
      idir=inr
C      ssqtm=t1(n1)+d2
      ssqtm=(sin((0.5*t11(n1)+2.0*dt)*0.01745329))**2  
      if(n1-30) 300,290,290
  290 nt30=30
      sq30=t1(30)
      go to 310
  300 nt30=0
      sq30=0
      if(n1-20) 320,310,310
  310 nt20=20
      sq20=t1(20)
      go to 330
  320 nt20=0
      sq20=0.0
  330 n2=0
      ih=0
      ik=0
      il=0
      n20=0
      n30=0
      nfi=0
      sqfi=t1(n1)
  340 call pw50
     >     (a11,a22,a33,a13,a12,a23,isym,ihm,ikm,ilm,ih,ik,il,ssqt,ia)
      if(ia) 350,510,350
  350 if(ssqt-ssqtm) 360,360,340
  360 if(isub) 370,380,370
  370 call ex(ih,ik,il,ij,numext,exsymb,ilatt)
      if(ij) 380,340,380
  380 call pw51(ssqt,t1,n1,t11,dt,ssqto,ia)
      if(ssqt-sqfi) 390,390,410
  390 call pw80(ssqt,t2,n2,iht,ikt,ilt,ih,ik,il,isym,inne)
      if(inne) 400,400,450
  400 nfi=nfi+1
  410 if(ssqt-sq30) 420,420,430
  420 n30=n30+1
  430 if(ssqt-sq20) 440,440,450
  440 n20=n20+1
  450 if(ia) 460,340,460
  460 if(inr) 490,470,490
  470 call pw53(ssqt,t2,t22,wg,n2,iht,ikt,ilt,iqe,ih,ik,il,isym,wt)
      if(ssqt) 480,340,340
  480 n2=n2+1
      if(n2-400) 340,670,670
  490 call pw53(ssqto,t2,t22,wg,n2,iht,ikt,ilt,iqe,ih,ik,il,isym,wt)
      if(ssqto) 500,340,340
  500 n2=n2+1
      if(n2-400) 340,510,510
  510 if(inr) 520,660,520
  520 if(iv) 530,540,530
  530 iv=iv-1
      go to 580
  540 call pw71(t2,wg,iht,ikt,ilt,iqe,isym,n2,a11,a22,a33,a13,a12,a23,
     >          v2,v22,t22,nekv)
      inr=0
      call pw63(a11,a22,a33,a13,a12,a23,wave,isym,a,b,c,al,be,ga)
      if(isym-6) 560,550,560
  550 call pw61(a,b,c,al,be,ga,s)
      go to 570
  560 call pw62(a,b,c,be,s,isym)
  570 continue
      call pw52(n,nekv,isym,v2,wave,amat,s,a,b,c,al,be,ga,
     >          da,db,dc,dal,dbe,dga,volm)
      go to 330
  580 call p55w(t2,wg,iqe,iht,ikt,ilt,n2,isym,amat,bmat,iekv)
      if(iekv.ne.0) goto 600
      write(iout,'(a)') ' Number of eq less than number of parameters.'
      inr=0
      go to 330
  600 n=1
      go to (650,640,630,640,620,610),isym
  610 n=n+2
  620 n=n+1
  630 n=n+1
  640 n=n+1
  650 continue
      call pw54(amat,n,bmat)
      a11=bmat(1,1)
      a22=bmat(2,1)
      a33=bmat(3,1)
      a13=bmat(4,1)
      a12=bmat(5,1)
      a23=bmat(6,1)
      write(iout,'(6f10.6)') a11,a22,a33,a13,a12,a23
      go to 330
  660 continue
      if (idir .gt. 1) then
         write(iout,661)
     >   ' Number of single indexed lines =',NEKV,
     >   ' Total number of lines          =',N1,
     >   ' A =',A,DA,' A   Alfa  =',AL,DAL,' Degrees',
     >   ' B =',B,DB,' A   Beta  =',BE,DBE,' Degrees',
     >   ' C =',C,DC,' A   Gamma =',GA,DGA,' Degrees',
     >   ' Unit cell volume =',VOLM,' A**3'
 661     format(2(' ',A,I5,/),3(' ',A,2F10.6,A,2F10.6,A,/),A,F10.2,A)
      endif
  670 call pw56(t1,t2,iht,ikt,ilt,n1,n2,iout,dt,wt,ip,wave,tagut,n20,
     >          n30,nfi,nt20,nt30,sq20,sq30,sqfi,m20,noind,t11,t22,ioup)
c      write(iout,'(a,f10.6)')
c     >      ' Summa(Delta(2th)**2)=',v22
c----------------------------------------------------------------------+
c --- For this time end of the calculations, close output file and     !
c     set the number of entries in the sine square theta table to      !
c     the original value (It is decreased to zero by some routines).   !
c     At last return to the main many.                                 !
c----------------------------------------------------------------------+
  777 inr = lernum
c
c --- if dflag was set then reset the value of iout to the list file,
c     otherwise finish as usual ...
c
      if (dflag) then
         iout = lulist
      else
         close (iout)
      endif
      goto 110
c
c --- This is the very end of this program.
c
  680 continue
      write (ludisp,'(a)') ' '
      write (ludisp,'(a)') ' Thank you for this session with PIRUM.'
      write (ludisp,'(a)') ' '
      end

      subroutine inform(ludisp,lukeys,linfo) 
      character info*70,ibeg*3,answer*1
      write(ludisp,'(a)') ' '
      numb_of_lines=0
      open (linfo,file='pirum.doc',status='old',err=10)
    1 read(linfo,'(2a)') info,ibeg
      numb_of_lines=numb_of_lines+1
      if(ibeg .eq. ' ') then
       write(ludisp,'(a)') info
       if(mod(numb_of_lines,24) .eq. 0) then
         write(ludisp,'(a,$)') ' **********PRESS RETURN********** '
         read(lukeys,'(a)') answer
           if(answer .eq. ' ') goto 1
           else
         goto 1
       endif
       goto 1
      endif
      goto 20
   10 write(ludisp,'(a)') ' '
      write(ludisp,'(a)') ' open file error on PIRUM.DOC '
      write(ludisp,'(a)') ' Is PIRUM.DOC in this directory ?'
      write(ludisp,'(a)') ' You may continue without INFO command. '
      write(ludisp,'(a,$)') ' **********PRESS RETURN********** '
      read(lukeys,'(a)') answer
      return
   20 write(ludisp,'(a,$)') ' **********PRESS RETURN********** '
      read(lukeys,'(a)') answer
      close(linfo)
      return
      end



            subroutine myconv (lukeys,ludisp,obstab,nobs,wave)
c
c --- this is a conversion routine to convert theta, d-values etc to
c     sine-square theta values...
c
      dimension obstab(1)
      character str80*80, answer*1

      write (ludisp,'(a)') ' '
      write (ludisp,'(a)') ' What type of data do you have ?'
      write (ludisp,'(a)') ' '
      write (ludisp,'(a)') '   1) Sine Square Theta.'
      write (ludisp,'(a)') '   2) 1/D**2.'
      write (ludisp,'(a)') '   3) Theta (deg)'
      write (ludisp,'(a)') '   4) 2Theta (deg)'
      write (ludisp,'(a)') '   5) d-values'
      write (ludisp,'(a)') ' '
      write (ludisp,'(a,$)') ' Enter your type here : '
      read  (lukeys,'(a)') str80

      if (str80 .eq. ' ') then
         write (ludisp,'(a)') ' No conversion done ...'
         return
      endif

C      read  (str80,*,iostat=istat) ityp
      call get_int(str80,istat,ityp)
      if (istat .ne. 0) then
         write (ludisp,'(a)')
     >   ' Error interpreting this answer as an integer ...'
         return
      endif

      if (ityp .lt. 1 .or. ityp .gt. 5) then
         write (ludisp,'(a)')
     >   ' Unknown data type, no conversion done ...'
         return
      endif

      if (ityp .eq. 1) then
         write (ludisp,'(a)')
     >   ' Type of data is correct for PIRUM, no conversion done ...'
         return
      endif
      
      write (ludisp,'(a)') ' '

      if (ityp .eq. 2) then
         write (ludisp,'(a)')
     >   ' You will convert from 1/d**2 to Sine Square (Theta)'

      elseif (ityp.eq.3) then
         write (ludisp,'(a)')
     >   ' You will convert from Theta (deg) to Sine Square (Theta)'

      elseif (ityp.eq.4) then
         write (ludisp,'(a)')
     >   ' You will convert from 2Theta (deg) to Sine Square (Theta)'

      elseif (ityp.eq.5) then
         write (ludisp,'(a)')
     >   ' You will convert from d-values to Sine Square (Theta)'

      endif

      write (ludisp,'(a)') ' '
      write (ludisp,'(a,$)') ' Is this okay ?  <Y/N> : '
      read  (lukeys,'(a)') answer

      if (answer .eq. 'n' .or. answer .eq. 'N') then
         write (ludisp,'(a)') ' No conversion done ...'
         return
      endif

      write (ludisp,'(a)') ' '
      do 1 i = 1,nobs
         sav = obstab(i)
         if (ityp .eq. 2) then
            obstab(i) = obstab(i)*0.25*wave*wave
            write (ludisp,'(i5,a,f12.6,a,f12.6)')
     >      i,':',sav,' ==> ',obstab(i)
         elseif (ityp .eq. 3) then
            obstab(i) = sqr(sin(torad(obstab(i))))
            write (ludisp,'(i5,a,f10.4,a,f12.6)')
     >      i,':',sav,' ==> ',obstab(i)
         elseif (ityp .eq. 4) then
            obstab(i) = sqr(sin(torad(0.5*obstab(i))))
            write (ludisp,'(i5,a,f10.4,a,f12.6)')
     >      i,':',sav,' ==> ',obstab(i)
         elseif (ityp .eq. 5) then
            obstab(i) = wave*wave*0.25/(obstab(i)*obstab(i))
            write (ludisp,'(i5,a,f10.4,a,f12.6)')
     >      i,':',sav,' ==> ',obstab(i)
         endif
 1    continue

      return
      end

c-----------------------------------------------------------------------

      function torad (x)
      torad = x * 0.01745329251994
      return
      end

c-----------------------------------------------------------------------
         
      function sqr(x)
      sqr=x*x
      return
      end

c-----------------------------------------------------------------------
         
      subroutine tyfile (lukeys,ludisp,luinp,str80)
c
c --- this routine types a file to the display.
c
      character str80*80, rad*80, answer*1

      if (str80 .eq. ' ') then
         write(ludisp,'(a,$)') ' Give name of file:'
         read(lukeys,'(a)') str80
      endif
      open (luinp,file=str80,status='old',iostat=istat)

      if (istat  .ne. 0) then
         write (ludisp,'(a,a,a)') 
     > ' Cannot open the file /',str80(istart(str80):lastch(str80)),'/'
         close (luinp)
         return
      endif
c
c --- ok, now print this file as long as something is left or the user 
c     want to stop, whichever comes first ...
c
      irad = 0
    1 continue
      read (luinp,'(a)',end=2) rad
      irad = irad+1
      if (rad .eq. ' ') then
         write(ludisp,'(a)') ' '
      else
         write(ludisp,'(a,a)') ' ',rad(1:lastch(rad))
      endif
      if (mod(irad,20) .eq. 0) then
         write(ludisp,'(a,$)') ' More ?  <Y[Def.]/N>'
         read(lukeys,'(a)') answer
         if (answer .eq. 'N' .or. answer .eq. 'n') then
            close (luinp)
            return
         endif
         irad = 0
      endif
      goto 1
c
c --- end of file encountered.
c
    2 continue
      close (luinp)

      return
      end

c-----------------------------------------------------------------------

      subroutine fnhelp (ludisp)
      write(ludisp,'(a)')
     >' INFO ......: General information about the program.'
      write(ludisp,'(a)')
     >' CELL ......: Set the real cell parameters.'
      write(ludisp,'(a)')
     >' CHANGE ....: Change some of the parameters (from a meny).'
      write(ludisp,'(a)')
     >' CONDITION .: Enter the reflexion condition routine.'
      write(ludisp,'(a)')
     >' CONVERT ...: Convert the spacing data.(def. sine sq. theta) '
      write(ludisp,'(a)')
     >' CREATE ....: Create a new PIRUM input file.'
      write(ludisp,'(a)')
     >' CYCLES ....: Set the number of refinement cycles.'
      write(ludisp,'(a)')
     >' DT ........: Set the DT parameter.'
      write(ludisp,'(a)')
C     >' D2 ........: Set the D2 parameter (not used ).'
     >' DISPLAY ...: Toggle between ouput on display or file.'
C      write(ludisp,'(a)')
C     >' ERASE .....: Erase the screen.'
      write(ludisp,'(a)')
     >' EXIT ......: Exit from this program.'
      write(ludisp,'(a)')
     >' HELP ......: Get some help. (This information..)'
      write(ludisp,'(a)')
     >' HKLMAX ....: Set the maximum H,K and L-values.'
      write(ludisp,'(a)')
     >' IP .(hkl)..: 0=normal, 1=hkl-file out, 2=hkl-file in '
C      write(ludisp,'(a)')
C     >' LATTICE ...: Set the type of lattice (P,I,F,A,B or C).'
      write(ludisp,'(a)')
     >' LOOK ......: Look at some parameters.'
      write(ludisp,'(a)')
     >' PREAD .....: Read complete data file. (IP=0 or IP=2 ?).'
      write(ludisp,'(a)')
     >' QREAD .....: Read data from keyboard/file.'
      write(ludisp,'(a)')
     >' RUN .......: Run the least square procedure.'
      write(ludisp,'(a)')
C       wt
     >' SYSTEM ....: Set the type of crystal system.'
      write(ludisp,'(a)')
     >' TITLE .....: Set the title string.'
      write(ludisp,'(a)')
     >' TYPE ......: Type a file on the display.'
      write(ludisp,'(a)')
     >' WAVE ......: Set the wavelength.'
      write(ludisp,'(a)')
     >' WT ........: Set the WT parameter (0=Hess , 1=unit weights.)'
      write(ludisp,'(a)')
      return
      end

c-----------------------------------------------------------------------

      subroutine crefil (lukeys,ludisp,iout,titstr,
     >                   ssqtab,rint,iantal,
     >                   dt,wt,ip,wave,ihm,ikm,ilm,isym,inr,isub,
     >                   exsymb,numext,ilatt,
     >                   a11,a22,a33,a13,a12,a23,iomv)
c
c --- This is the routine that enables the user to write a new complete
c     input file to PIRUM with all parameters.
c
      integer    exsymb(50,5),numext,ilatt
      character  fname*80, titstr*80, answer*1
      logical    flag

      dimension  ssqtab(200),rint(200)

      write (ludisp,'(a)') ' '
 110  write (ludisp,'(a,$)') ' Give the name of the file : '
      read (lukeys,'(a)') fname
      inquire(file=fname,exist=flag)
        if(flag) then
        write(ludisp,'(a,a,$)')
     >' This file already exists,',
     >' Do you wish to overwrite it ? (Y/N[Def.]'
        read(lukeys,'(a)') answer
          if(answer .ne. 'y' .and. answer .ne. 'Y') then
          goto 110
          endif
        endif
      if (fname .eq. ' ') then
         write (ludisp,'(a)') ' Blank filenames are not allowed !'
         goto 110
      endif
      open (unit=iout,file=fname,status='unknown',
     >      iostat=istat)
       if(istat .ne. 0) then
       write(ludisp,'(a,a,a)')
     > ' Error opening',fname,' try again..'
       goto 110
       endif
c
c --- Everything OK, go on ...
c
c
c --- title.
c
      i1 = last (titstr)
      if (i1 .lt. 1) then
         i1 = 1
      endif
      write (iout,'(a,a)') titstr(1:i1)
c
c --- read sine-square theta values and intensities ...
c
      do 90 i = 1,iantal
         write (iout,'(f15.6,f10.2)') ssqtab(i),rint(i)
   90 continue
c
c --- write a blank line to indicate that the sine-square theta values
c     end here ...
c
      write (iout,'(a)') ' '
c
c --- write the control parameters.
c
      write (iout,'(2f10.6,5x,i5,f10.6,6i5)')
     > dt,wt,ip,wave,ihm,ikm,ilm,isym,inr,isub

      if (iomv .eq. 1) then
         write (iout,'(6f10.4,i5)') a11,a22,a33,a13,a12,a23,iomv
      else
         write (iout,'(6f10.7,i5)') a11,a22,a33,a13,a12,a23,iomv
      endif

      write (iout,'(2i10)') numext,ilatt
      do 130 i = 1,numext
         write(iout,'(5i5)') (exsymb(i,j),j=1,5)
 130  continue

      close (iout)

      return
      end

c-----------------------------------------------------------------------

      SUBROUTINE P55W(T2,WG,IQE,IHT,IKT,ILT,NTOT,ISYM,A,B,IEKV)
C-----------------------------------------------------------------------
C     BILDA NORMALEKV. MATRISERNA A OCH B. ORIML.PROBLEM GER IEKV=0
C-----------------------------------------------------------------------
      DIMENSION T2(1),WG(1),IQE(1),IHT(1),IKT(1),ILT(1),A(6,6),B(6,1)
      DO 10 I=1,6
      DO 10 J=1,6
   10 A(I,J)=0.0
      DO 20 I=1,6
   20 B(I,1)=0.0
      IEKV=0
      GO TO(30,80,160,130,160,160),ISYM
   30 DO 50 I=1,NTOT
      IF(IQE(I))40,50,40
   40 U1=FLOAT(IHT(I)**2+IKT(I)**2+ILT(I)**2)
      A(1,1)=WG(I)*U1**2+A(1,1)
      B(1,1)=WG(I)*U1*T2(I)+B(1,1)
      IEKV=IEKV+1
   50 CONTINUE
      IF(IEKV-1)60,60,70
   60 IEKV=0
   70 RETURN
   80 DO 100 I=1,NTOT
      IF(IQE(I)) 90,100,90
   90 U1=FLOAT(IHT(I)**2+IKT(I)**2)
      A(1,1)=WG(I)*U1**2+A(1,1)
      A(1,2)=WG(I)*FLOAT(ILT(I)**2)*U1+A(1,2)
      A(2,2)=WG(I)*FLOAT(ILT(I))**4+A(2,2)
      B(1,1)=WG(I)*U1*T2(I)+B(1,1)
      B(2,1)=WG(I)*FLOAT(ILT(I)**2)*T2(I)+B(2,1)
      IEKV=IEKV+1
  100 CONTINUE
  110 IF(IEKV-2) 60,60,120
  120 A(2,1)=A(1,2)
      RETURN
  130 DO 150 I=1,NTOT
      IF(IQE(I))140,150,140
  140 U1=FLOAT(IHT(I)**2+IHT(I)*IKT(I)+IKT(I)**2)
      A(1,1)=WG(I)*U1**2+A(1,1)
      A(1,2)=WG(I)*U1*FLOAT(ILT(I)**2)+A(1,2)
      A(2,2)=WG(I)*FLOAT(ILT(I))**4+A(2,2)
      B(1,1)=WG(I)*U1*T2(I)+B(1,1)
      B(2,1)=WG(I)*FLOAT(ILT(I)**2)*T2(I)+B(2,1)
      IEKV=IEKV+1
  150 CONTINUE
      GO TO 110
  160 DO 200 I=1,NTOT
      IF(IQE(I)) 170,200,170
  170 U1=FLOAT(IHT(I)**2)
      U2=FLOAT(IKT(I)**2)
      U3=FLOAT(ILT(I)**2)
      A(1,1)=WG(I)*U1**2+A(1,1)
      A(1,2)=WG(I)*U1*U2+A(1,2)
      A(1,3)=WG(I)*U1*U3+A(1,3)
      B(1,1)=WG(I)*U1*T2(I)+B(1,1)
      B(2,1)=WG(I)*U2*T2(I)+B(2,1)
      B(3,1)=WG(I)*U3*T2(I)+B(3,1)
      A(2,2)=WG(I)*U2**2+A(2,2)
      A(2,3)=WG(I)*U2*U3+A(2,3)
      A(3,3)=WG(I)*U3**2+A(3,3)
      IEKV=IEKV+1
      GO TO(60,60,200,60,180,180),ISYM
  180 U4=FLOAT(IHT(I)*ILT(I))
      A(1,4)=WG(I)*U1*U4+A(1,4)
      A(2,4)=WG(I)*U2*U4+A(2,4)
      A(3,4)=WG(I)*U3*U4+A(3,4)
      A(4,4)=WG(I)*U4**2+A(4,4)
      B(4,1)=WG(I)*U4*T2(I)+B(4,1)
      GO TO(60,60,60,60,200,190),ISYM
  190 U5=FLOAT(IHT(I)*IKT(I))
      U6=FLOAT(IKT(I)*ILT(I))
      A(1,5)=WG(I)*U1*U5+A(1,5)
      A(1,6)=WG(I)*U1*U6+A(1,6)
      A(2,5)=WG(I)*U2*U5+A(2,5)
      A(2,6)=WG(I)*U2*U6+A(2,6)
      A(3,5)=WG(I)*U3*U5+A(3,5)
      A(3,6)=WG(I)*U3*U6+A(3,6)
      A(4,5)=WG(I)*U4*U5+A(4,5)
      A(4,6)=WG(I)*U4*U6+A(4,6)
      A(5,5)=WG(I)*U5**2+A(5,5)
      A(5,6)=WG(I)*U5*U6+A(5,6)
      A(6,6)=WG(I)*U6**2+A(6,6)
      B(5,1)=WG(I)*U5*T2(I)+B(5,1)
      B(6,1)=WG(I)*U6*T2(I)+B(6,1)
  200 CONTINUE
      A(2,1)=A(1,2)
      A(3,1)=A(1,3)
      A(3,2)=A(2,3)
      GO TO(60,60,210,60,220,220),ISYM
  210 IF(IEKV-3)60,60,70
  220 A(4,1)=A(1,4)
      A(4,2)=A(2,4)
      A(4,3)=A(3,4)
      GO TO(60,60,60,60,230,240),ISYM
  230 IF(IEKV-4)60,60,70
  240 A(5,1)=A(1,5)
      A(5,2)=A(2,5)
      A(5,3)=A(3,5)
      A(5,4)=A(4,5)
      A(6,1)=A(1,6)
      A(6,2)=A(2,6)
      A(6,3)=A(3,6)
      A(6,4)=A(4,6)
      A(6,5)=A(5,6)
      IF(IEKV-6)60,60,70
      END

c-----------------------------------------------------------------------

      subroutine getspc (lukeys,ludisp,luinp,ssqtab,rint,iantal,titstr)

      dimension ssqtab(1),rint(1)
      character answer*1,fname*80,titstr*80,st1*80,st2*80
c
c --- Read the sine square and intensity table ...
c
      write (ludisp,'(a,$)')
     > ' Do you want spacing-data from a file   Y/N  ? '
      read (lukeys,'(a)') answer
c
c --- input of spacing data from a file...
c
      if (answer .eq. 'Y' .or. answer .eq. 'y') then
         write (ludisp,'(a)') ' '
         write (ludisp,'(a,$)') ' Give the name of your file : '
         read (lukeys,'(a)') fname

         open (luinp,file=fname,status='old',iostat=istat)

         if (istat .ne. 0) then
            write (ludisp,'(a)') ' '
            write (ludisp,'(a)') ' Error opening your input file ....'
            return
         endif

         write (ludisp,'(a)') ' '
         write (ludisp,'(a,$)') 
     >   ' Is the first line a title line  <Y[Def.]/N> ? '
         read (lukeys,'(a)') answer

         if (answer .eq. 'Y' .or. answer .eq. 'y' .or.
     >       answer .eq. ' ') then
            read (luinp,'(a)') titstr
         endif
c
c ------ read one line, until EOF or a blank line is detected.
c
         i = 0
  170    continue
         read (luinp,'(a)',end=180) st1
         if (st1 .eq. ' ') then
            goto 180
         else
            call strip(st1)
            n = nextbl(st1)
            st2 = st1(n:80)
            st1(n:80) = ' '
C            read (st1,*,iostat=istat) sqtemp
            call get_real(st1,istat,sqtemp)
            if (istat .ne. 0) then
               write(ludisp,'(a)') ' Error during internal read.'
               goto 170
            endif
            if (sqtemp .gt. 0.0) then
               i = i+1
               ssqtab(i) = sqtemp
            else
               goto 180
            endif            
            if (st2 .eq. ' ') then
               rint(i) = 0.0
            else
               if (st2 .ne. ' ') then
                  call strip(st2)
                  call get_real(st2,istat,rint(i))
                  if (istat .ne. 0) then
                     write(ludisp,'(a)') ' Error during internal read.'
                     goto 170
                  endif
               else
                  rint(i) = 0.0
               endif
            endif
            goto 170
         endif
  180    continue
         iantal = i
         close (luinp)
c
c --- input of spacing data from the keyboard.
c
      elseif (answer .eq. 'N' .or. answer .eq. 'n') then
         write (ludisp,'(a)')
         write (ludisp,'(a,a)')
     >' Now you should type one sine square (theta) and one intensity ',
     > 'value / line. '
         write (ludisp,'(a)')
     > ' If other input data use CONVERT when input is finished '
         write (ludisp,'(a)')
     >' Finish by entering a blank line.'
         write (ludisp,'(a)') ' '
c
c ------ write a short text, requiring the next values.
c
         i = 0
  190    continue
         if (i .lt. 10) then
            write (ludisp,'(a,i1,a,i1,a,$)')
     >      ' Give SSQ:',I,'   and I:',I,' ---> '

         elseif (i .lt. 100) then
            write (ludisp,'(a,i2,a,i2,a,$)')
     >      ' Give SSQ:',I,'  and I:',I,' --> '

         else
            write (ludisp,'(a,i3,a,i3,a,$)')
     >      ' Give SSQ:',I,' and I:',I,' -> '

         endif
c
c ------ read ssqobs and possibly also the intensity...
c
         read (lukeys,'(a)') st1
         if (st1 .eq. ' ') then
            iantal = i
         else
            i = i+1
            call strip(st1)
            n = nextbl(st1)
            st2 = st1(n:80)
            st1(n:80) = ' '
            call get_real(st1,istat,ssqtab(i))
            if (istat .ne. 0) then
               write(ludisp,'(a)') 
     >         ' Error when reading SSQ-value, try again...'
               i = i-1
               goto 190
            endif
            if (st2 .eq. ' ') then
               rint(i) = 0.0
            else
               if (st2 .ne. ' ') then
                  call strip(st2)
                  call get_real(st2,istat,rint(i))
                  if (istat .ne. 0) then
                     write(ludisp,'(a)') 
     >               ' Error when reading intensity, try again...'
                     i = i-1
                     goto 190
                  endif
               else
                  rint(i) = 0.0
               endif
            endif
            goto 190
         endif
      else

         write (ludisp,'(a)') ' You must answer "Y" or "N" ...'
         return
      endif

      return
      end

c-----------------------------------------------------------------------

      subroutine pidnew (lukeys,ludisp,iin,titstr,
     >                   ssqtab,rint,iantal,
     >                   dt,wt,ip,wave,ihm,ikm,ilm,isym,inr,isub,
     >                   exsymb,numext,ilatt,
     >                   a1,a2,a3,a4,a5,a6,iomv)
c
c --- This is the routine that enables the user to read the parameters
c     to pirum in an interactive way ....
c
      integer    keys,ludisp
      character  answer*1, titstr*80, cdum*1, fname*80, st1*80,st2*80 
      logical    filflg, titflg
      integer    exsymb(50,5),numext

      dimension  ssqtab(200),rint(200)
C
C --- Read the sine square and intensity table ...
C
  120 continue
      write (ludisp,'(a)') ' '
      write (ludisp,'(a)')
     >' Now you should enter some sine square theta values.'
      write (ludisp,'(a)') ' '
      write (ludisp,'(a)')
     >' You have 2 possibilites to do this, either you read the values'
      write (ludisp,'(a)')
     >' from a file or otherwise you enter them from the keyboard ...'
      write (ludisp,'(a)') ' '     
      write (ludisp,'(a,a)')
     >' If you have SSQ-values on a file it can have a title line ',
     >'as the first line'
      write (ludisp,'(a,a)')
     >' and then a list of one SSQ-value and',
     >' Intesity-value on each line.'
      write (ludisp,'(a)') ' '
      write (ludisp,'(a)')
     >' The only stop signal in the file MUST be EOF.'
      write (ludisp,'(a)') ' '
  130 continue
c
      write (ludisp,'(a,$)')
     > ' Do you want SSQ-values from a file   Y/N  ? '
      read (lukeys,'(a)') answer
      if (answer .eq. 'Y' .or. answer .eq. 'y') then
         filflg = .true.
      elseif (answer .eq. 'N' .or. answer .eq. 'n') then
         filflg = .false.
      else
         goto 130
      endif

      if (filflg) then
         write (ludisp,'(a)') ' '
         write (ludisp,'(a,$)') ' Give the name of your file : '
         read (lukeys,'(a)') fname
         open (iin,file=fname,status='old',err=140)
         goto 150
  140    continue
         write (ludisp,'(a)') ' '
         write (ludisp,'(a)') ' Error opening your input file ....'
         goto 120
  150    continue
         write (ludisp,'(a)') ' '
  160    continue
         write (ludisp,'(a,$)') 
     >   ' Is the first line a title line  Y/N ? '
         read (lukeys,'(a)') answer
         if (answer .eq. 'Y' .or. answer .eq. 'y') then
            read (iin,'(a)') titstr
         elseif (answer .eq. 'N' .or. answer .eq. 'n') then
            write (ludisp,'(a)') ' '
            write (ludisp,'(a,$)') ' Title:'
            read (lukeys,'(a)') titstr
         else
            goto 160
         endif
c
c ------ read one line, until EOF or a blank line is detected.
c
         i = 0
  170    continue
         read (iin,'(a)',end=180) st1
         if (st1 .eq. ' ') then
            goto 180
         else
            call strip(st1)
            n = nextbl(st1)
            st2 = st1(n:80)
            st1(n:80) = ' '
            call get_real(st1,istat,sqtemp)
            if (istat .ne. 0) then
               write(ludisp,'(a)') ' Error during internal read.'
               goto 170
            endif
            if (sqtemp .gt. 0.0) then
               i = i+1
               ssqtab(i) = sqtemp
            else
               goto 180
            endif            
            if (st2 .eq. ' ') then
               rint(i) = 0.0
            else
               if (st2 .ne. ' ') then
                  call strip(st2)
                  call get_real(st2,istat,rint(i))
                  if (istat .ne. 0) then
                     write(ludisp,'(a)') ' Error during internal read.'
                     goto 170
                  endif
               else
                  rint(i) = 0.0
               endif
            endif
            goto 170
         endif
  180    continue
         iantal = i
         close (iin)

      else
         write (ludisp,'(a)')
         write (ludisp,'(a,a)')
     >' Now you should type one sine square (theta) and one intensity ',
     > 'value / line '
         write (ludisp,'(a)')
     >' Finish by entering a blank line.'
         write (ludisp,'(a)') ' '
c
c ------ write a short text, requiring the next values.
c
         i = 0
  190    continue
         if (i .lt. 10) then
            write (ludisp,'(a,i1,a,i1,a,$)')
     >      ' Give SSQ:',I,'   and I:',I,' ---> '

         elseif (i .lt. 100) then
            write (ludisp,'(a,i2,a,i2,a,$)')
     >      ' Give SSQ:',I,'  and I:',I,' --> '

         else
            write (ludisp,'(a,i3,a,i3,a,$)')
     >      ' Give SSQ:',I,' and I:',I,' -> '

         endif
c
c ------ read ssqobs and possibly also the intensity...
c
         read (lukeys,'(a)') st1
         if (st1 .eq. ' ') then
            iantal = i
            goto 210
         else
            i = i+1
            call strip(st1)
            n = nextbl(st1)
            st2 = st1(n:80)
            st1(n:80) = ' '
            call get_real(st1,istat,ssqtab(i))
            if (istat .ne. 0) then
               write(ludisp,'(a)') 
     >         ' Error when reading SSQ-value, try again...'
               i = i-1
               goto 190
            endif
            if (st2 .eq. ' ') then
               rint(i) = 0.0
            else
               if (st2 .ne. ' ') then
                  call strip(st2)
                  call get_real(st2,istat,rint(i))
                  if (istat .ne. 0) then
                     write(ludisp,'(a)') 
     >               ' Error when reading intensity, try again...'
                     i = i-1
                     goto 190
                  endif
               else
                  rint(i) = 0.0
               endif
            endif
            goto 190
         endif
      endif
c
c --- Now it's time for some control parameters ...
c
  210 continue
c
c --- Maybe a sorting of the data ?
c
      call sfixa (ssqtab,rint,iantal)
c
      write (ludisp,'(a)') ' '
  220 continue
C      write (ludisp,'(a,$)') ' Give D1, WT and D2 ......... : '
C      read (lukeys,*,err=220) d1,wt,d2
      write (ludisp,'(a,$)') ' Give DT and WT........... : '
      read (lukeys,*,err=220)
      write (ludisp,'(a)') ' '
  222 continue
      write (ludisp,'(a,$)') ' Give IP.........: '
      read (lukeys,*,err=222)  
      write (ludisp,'(a)') ' '
  230 continue
      write (ludisp,'(a,$)') ' Give the wavelenght ........... : '
      read (lukeys,*,err=230) wave
      write (ludisp,'(a)') ' '
  240 continue
      write (ludisp,'(a,$)') ' Give Hmax, Kmax and Lmax ...... : '
      read (lukeys,*,err=240) ihm,ikm,ilm
  250 continue
      write (ludisp,'(a)') ' '
      write (ludisp,'(a)') ' 1) Cubic.'
      write (ludisp,'(a)') ' 2) Tetragonal.'
      write (ludisp,'(a)') ' 3) Ortorombic.'
      write (ludisp,'(a)') ' 4) Hexagonal.'
      write (ludisp,'(a)') ' 5) Monoclinic.'
      write (ludisp,'(a)') ' 6) Triclinic.'
      write (ludisp,'(a)') ' '
  260 continue
      write (ludisp,'(a,$)') ' Give the symmetry code ........ : '
      read (lukeys,*,err=260) isym
      if (isym .ne. 1 .and. isym .ne. 2 .and. isym .ne. 3 .and.
     >    isym .ne. 4 .and. isym .ne. 5 .and. isym .ne. 6 ) then
         write (ludisp,'(a)') ' '
         write (ludisp,'(a)')
     >   ' Error, nonexistent code ... Try again !'
         goto 250
      endif
      write (ludisp,'(a)') ' '
  270 continue
      write (ludisp,'(a,$)') ' Number of refinement cycles ... : '
      read (lukeys,*,err=270) inr
      write (ludisp,'(a)') ' '

      answer = ' '
  280 continue
      if (answer .eq. 'y' .or. answer .eq. 'y') then
         isub = 1
         call exttop (lukeys,ludisp,numext,exsymb,ilatt,isub)      
      elseif (answer .eq. 'n' .or. answer .eq. 'n') then
         isub = 0
      else
         write (ludisp,'(a,$)')
     >   ' Do you want to specify reflexion conditions  Y/N ? '
         read (lukeys,'(a)') answer
         goto 280
      endif
c
c --- And now the cell parameters ...
c
      call chaaij (lukeys,ludisp,iomv,isym,a1,a2,a3,a4,a5,a6)

      return
      end

c-----------------------------------------------------------------------

      subroutine pilook (lukeys,ludisp,iin,iout,titstr,
     >                   ssqtab,rint,n1,
     >                   dt,wt,ip,wave,ihm,ikm,ilm,isym,inr,isub,
     >                   exsymb,numext,ilatt,
     >                   a11,a22,a33,a13,a12,a23,iomv)
c
c --- This is the routine that enables the user to look at the
c     parameters
c
      character  answer*1, fname*80, titstr*80, cdum*1
      logical    filflg, titflg
      integer    exsymb(50,5),numext

      dimension  ssqtab(200),rint(200)
c 
      CALL ERSSCR (ludisp)
C
C --- Title.
C
      I1 = LAST (TITSTR)
      IF (I1 .LT. 1) THEN
         I1 = 1
      ENDIF
      WRITE (ludisp,'(a,a)') ' Title:',TITSTR(1:I1)
      WRITE (ludisp,'(a)') ' '
C
C --- Sine square theta values.
C
      if (n1 .le. 0) then
         write (ludisp,'(a)') ' You do not have any SSQ-values.'
      else
         do 50 i = 1,n1
            write (ludisp,40) i,':',ssqtab(i),rint(i)
   40       format (i5,a,f10.6,f10.2)
   50    continue
      endif
      write (ludisp,'(a)') ' '
c
c --- d1,wt and d2. 
c
C      write (ludisp,60) ' D1 = ',d1,' WT = ',wt,' D2 = ',d2
C   60 format (3(a,f10.6))
      iput=ip
      if(ip .gt. 2) iput=2
      write (ludisp,60) ' DT = ',dt,' WT = ',wt,' IP = ',iput
   60 format (2(a,f10.6),a,i5)
      write (ludisp,'(a)') ' '
c
c --- wavelength.
c
      write (ludisp,70) ' Wavelenght = ',wave
   70 format (a,f10.7)
      write (ludisp,'(a)') ' '
c
c --- hmax,kmax and lmax.
c
      write (ludisp,80) ' H-max, K-max and L-max = ',ihm,ikm,ilm
   80 format (a,3i5)
      write (ludisp,'(a)') ' '
c
c --- system codes.
c
      if (isym .eq. 1) then
         write (ludisp,'(a)') ' Crystal system code = 1 = Cubic.'
      elseif (isym .eq. 2) then
         write (ludisp,'(a)') ' Crystal system code = 2 = Tetragonal.'
      elseif (isym .eq. 3) then
         write (ludisp,'(a)') ' Crystal system code = 3 = Ortorombic.'
      elseif (isym .eq. 4) then
         write (ludisp,'(a)') ' Crystal system code = 4 = Hexagonal.'
      elseif (isym .eq. 5) then
         write (ludisp,'(a)') ' Crystal system code = 5 = Monoclinic.'
      elseif (isym .eq. 6) then
         write (ludisp,'(a)') ' Crystal system code = 6 = Triclinic.'
      endif
      write (ludisp,'(a)') ' '
c
c --- number of refinement cycles.
c
      if(ip .lt. 2) then
      write (ludisp,'(a,i3,a)')' You have ',inr,' refinement cycles.'
      write (ludisp,'(a)') ' '
      endif
c
c --- extinction subroutine or not... (?)
c
      if (isub .eq. 1) then
         write (ludisp,'(a)') ' Reflexion conditions.'
         write (ludisp,'(a)') ' ---------------------'
         call extlis (lukeys,ludisp,numext,exsymb,ilatt)        
      else
        write (ludisp,'(a)')
     >  ' You do not have any reflexion conditions.'
      endif
      write (ludisp,'(a)') ' '
c
c --- cell parameters.
c
      if(ip .lt. 2) then
      write (ludisp,'(a,6f10.6)')
     > ' parameters=',a11,a22,a33,a13,a12,a23
      write (ludisp,'(a)') ' '
      endif
c
      return
      end

c-----------------------------------------------------------------------

      subroutine exttop (lukeys,ludisp,numext,exsymb,ilatt,isub)
c
      integer    exsymb(50,5),numext
      character  cmd*80

 1    continue
      write (ludisp,'(a,$)')   ' Condition>'
      read  (lukeys,'(a)') cmd
      if (cmd .eq. ' ') then
         goto 1
      else
         call strip(cmd)
         call upline(cmd)
      endif

      if (cmd .eq. 'ENTER' .or.
     >    cmd .eq. 'ENTE' .or.
     >    cmd .eq. 'ENT' .or.
     >    cmd .eq. 'EN' .or.
     >    cmd .eq. 'INPUT' .or.
     >    cmd .eq. 'INPU' .or.
     >    cmd .eq. 'INP' ) then
         call extinp (lukeys,ludisp,numext,exsymb,ilatt)

      elseif (cmd .eq. 'DELETE' .or.
     >        cmd .eq. 'DELET' .or.
     >        cmd .eq. 'DELE' .or.
     >        cmd .eq. 'DEL' .or.
     >        cmd .eq. 'DE' ) then
         call extdel (lukeys,ludisp,numext,exsymb,ilatt)

      elseif (cmd .eq. 'LIST' .or.
     >        cmd .eq. 'LIS' .or.
     >        cmd .eq. 'LI') then
         call extlis (lukeys,ludisp,numext,exsymb,ilatt)

      elseif (cmd .eq. 'INFO' .or.
     >        cmd .eq. 'INF') then
         call extinf (lukeys,ludisp)

      elseif (cmd .eq. 'EXIT' .or.
     >        cmd .eq. 'EXI' .or.
     >        cmd .eq. 'EX' ) then
         if (numext.eq.0.and.ilatt.eq.1) then
            isub = 0
         else
            isub = 1
         endif
         return

      elseif (cmd .eq. 'HELP' .or.
     >        cmd .eq. 'HEL' .or.
     >        cmd .eq. 'HE' ) then
         call exthlp (ludisp)

      else
         write (ludisp,'(a)') ' Unknown command.'

      endif
      goto 1

      end

c-----------------------------------------------------------------------

      subroutine extlis (lukeys,ludisp,numext,exsymb,ilatt)
c
c --- this routine list the reflexion conditions saved in the array
c     "EXSYMB"
c
      integer    exsymb(50,5)
      character  str*30

      write (ludisp,'(a)') ' '
c
c --- write the lattice type...
c
      if (ilatt .eq. 1) then
         write (ludisp,'(a)') ' Primitive lattice'
      elseif (ilatt .eq. 2) then
         write (ludisp,'(a)') ' I-centered lattice'
      elseif (ilatt .eq. 3) then
         write (ludisp,'(a)') ' F-centered lattice'
      elseif (ilatt .eq. 4) then
         write (ludisp,'(a)') ' A-centered lattice'
      elseif (ilatt .eq. 5) then
         write (ludisp,'(a)') ' B-centered lattice'
      elseif (ilatt .eq. 6) then
         write (ludisp,'(a)') ' C-centered lattice'
      endif

      do 1 i=1,numext
c
c ------ make sure that the string do contain only blanks...
c  
         str = '                              '
c
c ------ set reflexion type...
c
         if (exsymb(i,5) .eq. 1) then
            str(1:6) = 'hk0 : '
         elseif (exsymb(i,5) .eq. 2) then
            str(1:6) = 'h0l : '
         elseif (exsymb(i,5) .eq. 3) then
            str(1:6) = '0kl : '
         elseif (exsymb(i,5) .eq. 4) then
            str(1:6) = 'hhl : '
         elseif (exsymb(i,5) .eq. 5) then
            str(1:6) = 'h00 : '
         elseif (exsymb(i,5) .eq. 6) then
            str(1:6) = '0k0 : '
         elseif (exsymb(i,5) .eq. 7) then
            str(1:6) = '00l : '
         elseif (exsymb(i,5) .eq. 8) then
            str(1:6) = 'hh0 : '
         endif
c
c ------ set condition for existence of this reflexion.
c
         nplus = exsymb(i,1) + exsymb(i,2) + exsymb(i,3) - 1
         iptr=7

         if (exsymb(i,1) .eq. 1) then
            str(iptr:iptr) = 'h'
            iptr = iptr+1
         elseif (exsymb(i,1) .eq. 2) then
            str(iptr:iptr+1) = '2h'
            iptr = iptr+2
         endif
         if (exsymb(i,1) .ne. 0 .and. nplus .ge. 1) then
            str(iptr:iptr) = '+'
            iptr=iptr+1
            nplus=nplus-1
         endif

         if (exsymb(i,2) .eq. 1) then
            str(iptr:iptr) = 'k'
            iptr = iptr+1
         elseif (exsymb(i,2) .eq. 2) then
            str(iptr:iptr+1) = '2k'
            iptr = iptr+2
         endif
         if (exsymb(i,2) .ne. 0 .and. nplus .ge. 1) then
            str(iptr:iptr) = '+'
            iptr=iptr+1
            nplus=nplus-1
         endif

         if (exsymb(i,3) .eq. 1) then
            str(iptr:iptr) = 'l'
            iptr = iptr+1
         elseif (exsymb(i,3) .eq. 2) then
            str(iptr:iptr+1) = '2l'
            iptr = iptr+2
         endif

         str(iptr:iptr+2) = ' = '
         iptr=iptr+3

         if (exsymb(i,4) .eq. 2) then
            str(iptr:iptr+1) = '2n'
         elseif (exsymb(i,4) .eq. 3) then
            str(iptr:iptr+1) = '3n'
         elseif (exsymb(i,4) .eq. 4) then
            str(iptr:iptr+1) = '4n'
         elseif (exsymb(i,4) .eq. 6) then
            str(iptr:iptr+1) = '6n'
         endif
         write (ludisp,'(a,i2,a,a)') ' #',i,'  ',str
 1    continue

      write (ludisp,'(a)') ' '

      return
      end

c-----------------------------------------------------------------------

      subroutine extinp (lukeys,ludisp,numext,exsymb,ilatt)
c
c --- This routine decodes extiction conditions and stores the
c     codes of these to be used in other routines in the
c     integer array EXSYM (i,j)
c
c     The first index is only an ordinal number of the extinctions.
c     (There is NUMEXT of them ...)
c
c     EXSYMB (i,1) = #H
c     EXSYMB (i,2) = #K
c     EXSYMB (i,3) = #L
c     EXSYMB (i,4) = modulus number
c     EXSYMB (i,5) = reflexion type code.
c
      integer    exsymb(50,5)
      character  line*80,code*80,reftyp*10

 1    continue
      write (ludisp,'(a,$)')   ' Condition.Input>'
      read  (lukeys,'(a)') line

      if (line .eq. ' ') then
         return
      endif
c
c --- decode the given reflection code !
c
      call upline (line)
      call strip (line)      

      if (line .eq. 'HELP' .or.
     >    line .eq. 'HEL' .or.
     >    line .eq. 'HE' ) then
         write (ludisp,'(a)') ' '
         write (ludisp,'(a,/,a,a,/,a,/,a,/,a)')
     >   ' Give A,B,C,I or F for centered unit cells.',
     >   ' Other conditions should be entered on separate lines and',
     >   ' should contain',
     >   ' a reflexion type, a colon ":" and the actual condition.',
     >   ' Example:   0K0 : K=2N',
     >   ' End input with a blank line.'
         write (ludisp,'(a)') ' '
         goto 1
      endif
c
c --- check first if a lattice centering symbol was given.
c
      if (line .eq. 'P') then
         ilatt = 1
         write (ludisp,'(a)') ' '
         write (ludisp,'(a)') ' Primitve lattices used.'
         write (ludisp,'(a)') ' '
         goto 1
      elseif (line .eq. 'I') then
         ilatt = 2
         write (ludisp,'(a)') ' '
         write (ludisp,'(a)') ' I-centered lattice used.'
         write (ludisp,'(a)') ' '
         goto 1
      elseif (line .eq. 'F') then
         ilatt = 3
         write (ludisp,'(a)') ' '
         write (ludisp,'(a)') ' F-centered lattice used.'
         write (ludisp,'(a)') ' '
         goto 1
      elseif (line .eq. 'A') then
         ilatt = 4
         write (ludisp,'(a)') ' '
         write (ludisp,'(a)') ' A-centered lattice used.'
         write (ludisp,'(a)') ' '
         goto 1
      elseif (line .eq. 'B') then
         ilatt = 5
         write (ludisp,'(a)') ' '
         write (ludisp,'(a)') ' B-centered lattice used.'
         write (ludisp,'(a)') ' '
         goto 1
      elseif (line .eq. 'C') then
         ilatt = 6
         write (ludisp,'(a)') ' '
         write (ludisp,'(a)') ' C-centered lattice used.'
         write (ludisp,'(a)') ' '
         goto 1
      endif
c
c --- not lattice centring, but another type of symbol
c
      code = ' '
      j = 1
      do 2 i = 1,lastch(line)
         if (line(i:i) .ne. ' ') then
            code (j:j) = line(i:i)
            j = j+1
         endif
 2    continue
c
c --- now string "code" contains REF-TYPE, :, REFLECTION-CODE 
c     for example: 0K0:K=2N
c
      reftyp = ' '
      i = index(code,':')
      if (i .eq. 0) then
         write (ludisp,'(a)') ' '
         write (ludisp,'(a)') 
     >   ' Cannot decode this string, please try again...'
         write (ludisp,'(a)') ' '
         goto 1
      else
         reftyp = code(1:i-1)
         code (1:i) = ' '
      endif

      call strip (code)
c
c --- check hk0 reflexions.
c
      if (reftyp .eq. 'HK0') then
         if (code .eq. 'H=2N') then
            numext = numext+1
            exsymb(numext,1) = 1
            exsymb(numext,2) = 0
            exsymb(numext,3) = 0
            exsymb(numext,4) = 2
            exsymb(numext,5) = 1
         elseif (code .eq. 'K=2N') then
            numext = numext+1
            exsymb(numext,1) = 0
            exsymb(numext,2) = 1
            exsymb(numext,3) = 0
            exsymb(numext,4) = 2
            exsymb(numext,5) = 1
         elseif (code .eq. 'H+K=2N') then
            numext = numext+1
            exsymb(numext,1) = 1
            exsymb(numext,2) = 1
            exsymb(numext,3) = 0
            exsymb(numext,4) = 2
            exsymb(numext,5) = 1
         elseif (code .eq. 'H+K=4N') then
            numext = numext+1
            exsymb(numext,1) = 1
            exsymb(numext,2) = 1
            exsymb(numext,3) = 0
            exsymb(numext,4) = 4
            exsymb(numext,5) = 1
         else
            write (ludisp,'(a)') ' '
            write (ludisp,'(a)')
     >      ' Unknown reflection code for reflexion type HK0.'
            write (ludisp,'(a)') ' '
         endif
c
c --- check h0l reflexions.
c
      elseif (reftyp .eq. 'H0L') then
         if (code .eq. 'H=2N') then
            numext = numext+1
            exsymb(numext,1) = 1
            exsymb(numext,2) = 0
            exsymb(numext,3) = 0
            exsymb(numext,4) = 2
            exsymb(numext,5) = 2
         elseif (code .eq. 'L=2N') then
            numext = numext+1
            exsymb(numext,1) = 0
            exsymb(numext,2) = 0
            exsymb(numext,3) = 1
            exsymb(numext,4) = 2
            exsymb(numext,5) = 2
         elseif (code .eq. 'H+L=2N') then
            numext = numext+1
            exsymb(numext,1) = 1
            exsymb(numext,2) = 0
            exsymb(numext,3) = 1
            exsymb(numext,4) = 2
            exsymb(numext,5) = 2
         elseif (code .eq. 'H+L=4N') then
            numext = numext+1
            exsymb(numext,1) = 1
            exsymb(numext,2) = 0
            exsymb(numext,3) = 1
            exsymb(numext,4) = 4
            exsymb(numext,5) = 2
         else
            write (ludisp,'(a)') ' '
            write (ludisp,'(a)')
     >      ' Unknown reflection code for reflexion type H0L.'
            write (ludisp,'(a)') ' '
         endif
c
c --- check 0kl reflexions.
c
      elseif (reftyp .eq. '0KL') then
         if (code .eq. 'K=2N') then
            numext = numext+1
            exsymb(numext,1) = 0
            exsymb(numext,2) = 1
            exsymb(numext,3) = 0
            exsymb(numext,4) = 2
            exsymb(numext,5) = 3
         elseif (code .eq. 'L=2N') then
            numext = numext+1
            exsymb(numext,1) = 0
            exsymb(numext,2) = 0
            exsymb(numext,3) = 1
            exsymb(numext,4) = 2
            exsymb(numext,5) = 3
         elseif (code .eq. 'K+L=2N') then
            numext = numext+1
            exsymb(numext,1) = 0
            exsymb(numext,2) = 1
            exsymb(numext,3) = 1
            exsymb(numext,4) = 2
            exsymb(numext,5) = 3
         elseif (code .eq. 'K+L=4N') then
            numext = numext+1
            exsymb(numext,1) = 0
            exsymb(numext,2) = 1
            exsymb(numext,3) = 1
            exsymb(numext,4) = 4
            exsymb(numext,5) = 3
         else
            write (ludisp,'(a)') ' '
            write (ludisp,'(a)')
     >      ' Unknown reflection code for reflexion type 0KL.'
            write (ludisp,'(a)') ' '
         endif
c
c --- check hhl reflexions.
c
      elseif (reftyp .eq. 'HHL') then
         if (code .eq. 'L=2N') then
            numext = numext+1
            exsymb(numext,1) = 0
            exsymb(numext,2) = 0
            exsymb(numext,3) = 1
            exsymb(numext,4) = 2
            exsymb(numext,5) = 4
         elseif (code .eq. '2H+L=4N') then
            numext = numext+1
            exsymb(numext,1) = 2
            exsymb(numext,2) = 0
            exsymb(numext,3) = 1
            exsymb(numext,4) = 4
            exsymb(numext,5) = 4
         else
            write (ludisp,'(a)') ' '
            write (ludisp,'(a)')
     >      ' Unknown reflection code for reflexion type HHL.'
            write (ludisp,'(a)') ' '
         endif
c
c --- check h00 reflexions.
c
      elseif (reftyp .eq. 'H00') then
         if (code .eq. 'H=2N') then
            numext = numext+1
            exsymb(numext,1) = 1
            exsymb(numext,2) = 0
            exsymb(numext,3) = 0
            exsymb(numext,4) = 2
            exsymb(numext,5) = 5
         elseif (code .eq. 'H=4N') then
            numext = numext+1
            exsymb(numext,1) = 1
            exsymb(numext,2) = 0
            exsymb(numext,3) = 0
            exsymb(numext,4) = 2
            exsymb(numext,5) = 5
         else
            write (ludisp,'(a)') ' '
            write (ludisp,'(a)')
     >      ' Unknown reflection code for reflexion type H00.'
            write (ludisp,'(a)') ' '
         endif
c
c --- check 0k0 reflexions.
c
      elseif (reftyp .eq. '0K0') then
         if (code .eq. 'K=2N') then
            numext = numext+1
            exsymb(numext,1) = 0
            exsymb(numext,2) = 1
            exsymb(numext,3) = 0
            exsymb(numext,4) = 2
            exsymb(numext,5) = 6
         elseif (code .eq. 'K=4N') then
            numext = numext+1
            exsymb(numext,1) = 0
            exsymb(numext,2) = 1
            exsymb(numext,3) = 0
            exsymb(numext,4) = 4
            exsymb(numext,5) = 6
         else
            write (ludisp,'(a)') ' '
            write (ludisp,'(a)')
     >      ' Unknown reflection code for reflexion type 0K0.'
            write (ludisp,'(a)') ' '
         endif
c
c --- check 00l reflexions.
c
      elseif (reftyp .eq. '00L') then
         if (code .eq. 'L=2N') then
            numext = numext+1
            exsymb(numext,1) = 0
            exsymb(numext,2) = 0
            exsymb(numext,3) = 1
            exsymb(numext,4) = 2
            exsymb(numext,5) = 7
         elseif (code .eq. 'L=3N') then
            numext = numext+1
            exsymb(numext,1) = 0
            exsymb(numext,2) = 0
            exsymb(numext,3) = 1
            exsymb(numext,4) = 3
            exsymb(numext,5) = 7
         elseif (code .eq. 'L=4N') then
            numext = numext+1
            exsymb(numext,1) = 0
            exsymb(numext,2) = 0
            exsymb(numext,3) = 1
            exsymb(numext,4) = 4
            exsymb(numext,5) = 7
         elseif (code .eq. 'L=6N') then
            numext = numext+1
            exsymb(numext,1) = 0
            exsymb(numext,2) = 0
            exsymb(numext,3) = 1
            exsymb(numext,4) = 6
            exsymb(numext,5) = 7
         else
            write (ludisp,'(a)') ' '
            write (ludisp,'(a)')
     >      ' Unknown reflection code for reflexion type 00L.'
            write (ludisp,'(a)') ' '
         endif
c
c --- check hh0 reflexions.
c
      elseif (reftyp .eq. 'HH0') then
         if (code .eq. 'H=2N') then
            numext = numext+1
            exsymb(numext,1) = 1
            exsymb(numext,2) = 0
            exsymb(numext,3) = 0
            exsymb(numext,4) = 2
            exsymb(numext,5) = 8
         else
            write (ludisp,'(a)') ' '
            write (ludisp,'(a)')
     >      ' Unknown reflection code for reflexion type HH0.'
            write (ludisp,'(a)') ' '
         endif

      elseif (reftyp .eq. 'HKL') then
         write (ludisp,'(a)') ' '
         write (ludisp,'(a,a)')
     >   ' Use the A,B,C,F,I or P command to specify',
     >   ' general reflexion conditions.'
         write (ludisp,'(a)') ' '

      else
         write (ludisp,'(a)') ' '
         write (ludisp,'(a,a)') ' Unknown reflexion type: ',reftyp
         write (ludisp,'(a)') ' '

      endif
      goto 1

      end

c-----------------------------------------------------------------------

      subroutine extinf (lukeys,ludisp)
c
c --- this routine gives some information about the use of extinction
c     conditions, more specific it lists the allowed "codes".
c
      write (ludisp,'(a)') ' '
      write (ludisp,'(a,a)')
     >' A reflexion condition consists of a reflexion type,',
     >' to indicate when'
      write (ludisp,'(a,a)')
     >' the reflexion condition is applicable, a colon ":" and',
     >' finally the '
      write (ludisp,'(a)')
     >' actual reflexion condition.'
      write (ludisp,'(a)')
     >' A list of the allowed reflection conditions is given below.'
      write (ludisp,'(a)')
     >' '
      write (ludisp,'(a)')
     >'       Refl.   Possible reflection conditions'
      write (ludisp,'(a)')
     >'        ---    ------------------------------'
      write (ludisp,'(a)')
     >'        HK0 :  H=2N    K=2N    H+K=2N  H+K=4N'
      write (ludisp,'(a)')
     >'        H0L :  H=2N    L=2N    H+L=2N  H+L=4N'
      write (ludisp,'(a)')
     >'        0KL :  K=2N    L=2N    K+L=2N  K+L=4N'
      write (ludisp,'(a)')
     >'        HHL :  L=2N    2H+L=4N'
      write (ludisp,'(a)')
     >'        H00 :  H=2N    H=4N'
      write (ludisp,'(a)')
     >'        0K0 :  K=2N    K=4N'
      write (ludisp,'(a)')
     >'        00L :  L=2N    L=3N    L=4N    L=6N'
      write (ludisp,'(a)')
     >'        HH0 :  H=2N'
      write (ludisp,'(a)')
     >' '
      write (ludisp,'(a,a)')
     >' As an example consider the case of a 2-fold screw-axis',
     >' along b, then' 
      write (ludisp,'(a)')
     >' the reflexion condition should be:     OKO : K=2N'
      write (ludisp,'(a,a)')
     >' Only one condition can be entered on each line and should',
     >' conform'
      write (ludisp,'(a)')
     >' to the standard given in the International Tables.'
      write (ludisp,'(a)') ' '

      return
      end

c-----------------------------------------------------------------------

      subroutine exthlp(i)
 1    format (' ',a)
 2    format (' ',a,a)
      write(i,1)
     >'DELETE .......: Delete a condition from the list.'
      write(i,1)
     >'EXIT .........: Return to PIRUM top-level.'
      write(i,2)
     >'INFO .........:',
     >' Get some information how to use reflexion conditions.'
      write(i,1)
     >'INPUT ........: Enter one (or more) reflexion condition(s).'
      write(i,1)
     >'HELP .........: Get some help (this info).'
      write(i,1)
     >'LIST .........: List the saved reflexion conditions.'
      return
      end

c-----------------------------------------------------------------------

      subroutine extdel (lukeys,ludisp,numext,exsymb,ilatt)

      integer    exsymb(50,5)
c
      write (ludisp,'(a,$)') ' Give number of condition to delete: '
      read (lukeys,*,iostat=istat) num

      if (istat .eq. 0) then
         if (num .ge. 1 .and. num .le. numext) then
            do 1 i = num,numext-1
               do 2 j = 1,5
                  exsymb(i,j) = exsymb(i+1,j)
 2             continue
 1          continue
            numext = numext-1
         endif
      endif
 
      return
      end

c-----------------------------------------------------------------------

      logical function extinct (h,k,l,numext,ixs,ilatt)
c
c --- this checks if a reflexion is allowed, according to lattice
c     centering and other entered symmetry restrictions.
c
      integer h,k,l
      logical symmok,even,odd,lattok
      integer ixs(50,5),numext
c
c --- check lattice exticntions.
c
      if (ilatt .eq. 1) then
         lattok = .true.
      elseif (ilatt.eq.2.and.even(h+k+l)) then
         lattok = .true.
      elseif (ilatt.eq.3.and.even(h+k).and.even(h+l).and.even(k+l)) then
         lattok = .true.
      elseif (ilatt.eq.4.and.even(k+l)) then
         lattok = .true.
      elseif (ilatt.eq.5.and.even(h+l)) then
         lattok = .true.
      elseif (ilatt.eq.6.and.even(h+k)) then
         lattok = .true.
      else
         lattok = .false.
      endif
c
c --- check other types of extinctions than lattice extintions.
c     extinction codes stored in the array ixs(i,j)
c
      symmok = .true.

      do 1 i = 1,numext
         isum = h*ixs(i,1) + k*ixs(i,2) + l*ixs(i,3)

c ------ HK0 (type = 1)
         if (l.eq.0 .and.
     >       ixs(i,5).eq.1 .and. mod(isum,ixs(i,4)).ne.0)then
            symmok = .false.
            goto 2

c ------ H0L (type = 2)
         elseif (k.eq.0 .and.
     >           ixs(i,5).eq.2 .and. mod(isum,ixs(i,4)).ne.0)then
            symmok = .false.
            goto 2

c ------ 0KL (type = 3)
         elseif (h.eq.0 .and.
     >           ixs(i,5).eq.3 .and. mod(isum,ixs(i,4)).ne.0)then
            symmok = .false.
            goto 2

c ------ HHL (type = 4)
         elseif (k.eq.h .and.
     >           ixs(i,5).eq.4 .and. mod(isum,ixs(i,4)).ne.0)then
            symmok = .false.
            goto 2

c ------ H00 (type = 5)
         elseif (k.eq.0 .and. l.eq.0 .and.
     >           ixs(i,5).eq.5 .and. mod(isum,ixs(i,4)).ne.0)then
            symmok = .false.
            goto 2

c ------ 0K0 (type = 6)
         elseif (h.eq.0 .and. l.eq.0 .and.
     >           ixs(i,5).eq.6 .and. mod(isum,ixs(i,4)).ne.0)then
            symmok = .false.
            goto 2

c ------ 00L (type = 7)
         elseif (h.eq.0 .and. k.eq.0 .and.
     >           ixs(i,5).eq.7 .and. mod(isum,ixs(i,4)).ne.0)then
            symmok = .false.
            goto 2

c ------ HH0 (type = 8)
         elseif (k.eq.h .and. l.eq.0 .and.
     >           ixs(i,5).eq.8 .and. mod(isum,ixs(i,4)).ne.0)then
            symmok = .false.
            goto 2

         endif
 1    continue
c
c --- set return value of this function.
c
 2    continue
      if (lattok .and. symmok) then
         extinct = .false.
      else
         extinct = .true.
      endif

      return
      end
    
c-----------------------------------------------------------------------

      subroutine pircha (lukeys,ludisp,iin,iout,titstr,
     >                   ssqtab,rint,iantal,
     >                   dt,wt,ip,wave,ihm,ikm,ilm,isym,inr,isub,
     >                   exsymb,numext,ilatt,
     >                   a1,a2,a3,a4,a5,a6,iomv)
c
c --- This is the routine that enables the user to change the values
c     of some of the parameters.
c     Choose an item from a meny and set the new value ...
c
      character  answer*1, fname*80, titstr*80
      integer    exsymb(50,5),numext,ilatt

      dimension  ssqtab(200),rint(200)

 4711 continue
      call ersscr (ludisp)
      write(ludisp,'(a)') ' '
      write(ludisp,'(a)') ' What do you want to do ?'
      write(ludisp,'(a)') ' ----------------------------------'
      write(ludisp,'(a)') ' 1) Change the TITLE.'
      write(ludisp,'(a)') ' 2) Change some of the SSQ-values.'
      write(ludisp,'(a)') ' 3) Change DT.'
      write(ludisp,'(a)') ' 4) Change WT.'
      write(ludisp,'(a)') ' 5) Change IP.'
      write(ludisp,'(a)') ' 6) Change the wavelength.'
      write(ludisp,'(a)') ' 7) Change H-max, K-max and L-max.'
      write(ludisp,'(a)') ' 8) Change the number of refinement cycles.'
      write(ludisp,'(a)') ' 9) Change the symmetry code.'
      write(ludisp,'(a)')
     > ' A) Change the value of ISUB (Reflexion conditions)'
      write(ludisp,'(a)') ' B) Change the cell parameters.'
      write(ludisp,'(a)') ' C) Go back to the main meny.'
      write(ludisp,'(a)') ' '
      write(ludisp,'(a,$)') ' Type your answer here ---> '
      read (lukeys,'(a)') answer
      write(ludisp,'(a)') ' '

      if (answer .eq. '1') then
         call chatit(lukeys,ludisp,titstr)

      elseif (answer .eq. '2') then
         call pirssq (lukeys,ludisp,ssqtab,rint,iantal)

      elseif (answer .eq. '3') then
         call chad1(lukeys,ludisp,dt)

      elseif (answer .eq. '4') then
         call chassqtl(lukeys,ludisp,wt)

      elseif (answer .eq. '5') then
         call chad2(lukeys,ludisp,ip)
C         call chad2(lukeys,ludisp,d2)
      elseif (answer .eq. '6') then
         call chawave(lukeys,ludisp,wave)

      elseif (answer .eq. '7') then
         call chahklmax(lukeys,ludisp,ihm,ikm,ilm)

      elseif (answer .eq. '8') then
         call chainr(lukeys,ludisp,inr)

      elseif (answer .eq. '9') then
         call chasys (lukeys,ludisp,isym)

      elseif (answer .eq. 'A' .or. answer .eq. 'a') then
         call chacond (lukeys,ludisp,isub,numext,exsymb,ilatt)

      elseif (answer .eq. 'B' .or. answer .eq. 'b') then
         call chaaij (lukeys,ludisp,iomv,isym,a1,a2,a3,a4,a5,a6)

      elseif (answer .eq. 'C' .or. answer .eq. 'c') then
         goto 9999

      endif
      goto 4711

 9999 continue
      return
      end


c-----------------------------------------------------------------------

      subroutine chatit(lukeys,ludisp,titstr)
      character titstr*80
      write(ludisp,'(a,$)') ' Title:'
      read (lukeys,'(a)') titstr
      return
      end

c-----------------------------------------------------------------------

      subroutine chad1(lukeys,ludisp,dt)
      character str*80
      write (ludisp,'(a,f8.5,a,$)') ' Give DT [',dt,'] :'
      read (lukeys,'(a)') str
      if (str.eq.' ') then
         write(ludisp,'(a)') ' No change done ...'
         return
      endif
      call strip(str)
      call get_real(str,istat,temp)
      if (istat .ne. 0) then
         write(ludisp,'(a)')
     >   ' Error during numeric read, no change done ...'
      else
         dt = temp
      endif
      return
      end

c-----------------------------------------------------------------------

      subroutine chassqtl(lukeys,ludisp,wt)
      character str*80
      write (ludisp,'(a,f8.5,a,$)') ' Give WT [',wt,'] :'
      read (lukeys,'(a)') str
      if (str.eq.' ') then
         write(ludisp,'(a)') ' No change done ...'
         return
      endif
      call strip(str)
      call get_real(str,istat,temp)
      if (istat .ne. 0) then
         write(ludisp,'(a)')
     >   ' Error during numeric read, no change done ...'
      else
         wt = temp
      endif
      return
      end

c-----------------------------------------------------------------------

      subroutine chad2(lukeys,ludisp,ip)
      character str*80
      write (ludisp,'(a,i5,a,$)') ' Give IP [',ip,'] :'
      read (lukeys,'(a)') str
      if (str.eq.' ') then
         write(ludisp,'(a)') ' No change done ...'
         return
      endif
      call strip(str)
      call get_int(str,istat,itemp)
      if (istat .ne. 0) then
         write(ludisp,'(a)')
     >   ' Error during numeric read, no change done ...'
      else
         ip = itemp
      endif
         if (IP .eq. 2) then
         write(ludisp,'(a)') ' Use PREAD to read indexed data. '
         call wait(lukeys,ludisp)
         endif
C      call get_real(str,istat,temp)
C      if (istat .ne. 0) then
C         write(ludisp,'(a)')
C     >   ' Error during numeric read, no change done ...'
C      else
C         d2 = temp
C      endif
      return
      end

c-----------------------------------------------------------------------

      subroutine chawave(lukeys,ludisp,wave)
      character str*80
      write (ludisp,'(a,f10.6,a,$)') ' Give wave [',wave,'] :'
      read (lukeys,'(a)') str
      if (str.eq.' ') then
         write(ludisp,'(a)') ' No change done ...'
         return
      endif
      call strip(str)
      call get_real(str,istat,temp)
      if (istat .ne. 0) then
         write(ludisp,'(a)')
     >   ' Error during numeric read, no change done ...'
      else
         wave = temp
      endif
      return
      end

c-----------------------------------------------------------------------

      subroutine chahklmax(lukeys,ludisp,ihm,ikm,ilm)
      character st1*80,st2*80,st3*80
      write(ludisp,'(a,3i4,a,$)')
     > ' Give H-max, K-max and L-max [',ihm,ikm,ilm,'] :'
      read (lukeys,'(a)') st1
      if (st1.eq.' ') then
         write(ludisp,'(a)') ' No change done ...'
         return
      endif
      call strip(st1)
      n = nextbl(st1)
      st2 = st1(n:80)
      st3 = ' '
      st1(n:80) = ' '
      if (st2.ne.' ') then
         call strip(st2)
         n = nextbl(st2)
         st3 = st2(n:80)
         st2(n:80) = ' '
         if (st3.ne.' ') then
            call strip(st3)
         endif
      endif
      i2 = ikm
      i3 = ilm
C      read (st1,*,iostat=istat1) i1
      call get_int(st1,istat1,i1)
      if (st2 .ne. ' ') then
C         read (st2,*,iostat=istat2) i2
         call get_int(st2,istat2,i2)
         if (st3 .ne. ' ') then
C            read (st3,*,iostat=istat3) i3
         call get_int(st3,istat3,i3)
         endif
      endif
      if (istat1.ne.0 .or. istat2.ne.0 .or. istat3.ne.0) then
         write(ludisp,'(a)')
     >   ' Error during numeric read, no change done ...'
      else
         ihm = i1
         ikm = i2
         ilm = i3
      endif

      return
      end

c-----------------------------------------------------------------------

      subroutine chainr(lukeys,ludisp,inr)
      character str*80
      write (ludisp,'(a,i4,a,$)')
     > ' Give number of refinement cycles [',inr,'] :'
      read (lukeys,'(a)') str
      if (str.eq.' ') then
         write(ludisp,'(a)') ' No change done ...'
         return
      endif
      call strip (str)
      call get_int(str,istat,itemp)
      if (istat .ne. 0) then
         write(ludisp,'(a)')
     >   ' Error during numeric read, no change done ...'
      else
         inr = itemp
      endif
      return
      end

c-----------------------------------------------------------------------

C      subroutine chalatt(lukeys,ludisp,ilatt)
C      character answer*1
C     write(ludisp,'(a,$)') ' Enter lattice-type (P,I,F,A,B or C):'
C      read (lukeys,'(a)') answer
C      if (answer .eq. 'p' .or. answer .eq. 'P') then
C         ilatt = 1
C      elseif (answer .eq. 'i' .or. answer .eq. 'I') then
C         ilatt = 2
C      elseif (answer .eq. 'f' .or. answer .eq. 'F') then
C         ilatt = 3
C      elseif (answer .eq. 'a' .or. answer .eq. 'A') then
C         ilatt = 4
C      elseif (answer .eq. 'b' .or. answer .eq. 'B') then
C         ilatt = 5
C      elseif (answer .eq. 'c' .or. answer .eq. 'C') then
C         ilatt = 6
C      endif
C      return
C      end

c-----------------------------------------------------------------------

      subroutine chasys (lukeys,ludisp,isym)  
      character str*80
      write(ludisp,'(a)') ' Available crystal systems.'
      write(ludisp,'(a)') ' '
      write(ludisp,'(a)') '   1) Cubic.'
      write(ludisp,'(a)') '   2) Tetragonal.'
      write(ludisp,'(a)') '   3) Ortorombic.'
      write(ludisp,'(a)') '   4) Hexagonal.'
      write(ludisp,'(a)') '   5) Monoclinic.'
      write(ludisp,'(a)') '   6) Triclinic.'
      write(ludisp,'(a)') ' '
      write(ludisp,'(a,$)')
     >' Give crystal system code (RETURN = No change): '
      read (lukeys,'(a)') str
      if (str.eq.' ') then
         write(ludisp,'(a)') ' No change done ...'
         return
      endif
      call strip(str)
      call get_int(str,istat,itemp)
      if (istat .ne. 0) then
         write (ludisp,'(a)')
     >   ' Error during numeric read, no change done ...'
         return
      endif
      if (itemp .lt. 0 .or. itemp .gt. 6) then
         write (ludisp,'(a)') ' '
         write (ludisp,'(a)') ' Value not allowed, try again ...'
      else
         if (itemp .ne. 0) then
            isym = itemp
         endif
      endif
      return
      end

c-----------------------------------------------------------------------

      subroutine chacond (lukeys,ludisp,isub,numext,exsymb,ilatt)
      integer exsymb(50,5)
      write(ludisp,'(a)') ' '
      write(ludisp,'(a)')' ISUB = 0  --> Reflexion conditions not used.'
      write(ludisp,'(a)')' ISUB = 1  --> Reflexion conditions used.'
      write(ludisp,'(a)') ' '
      write(ludisp,'(a,$)') ' Give new value of ISUB:'
      read(lukeys,*) itemp
      if (itemp .ne. 0 .and. itemp .ne. 1) then
         write (ludisp,'(a)') ' Value not allowed...'
      else
         isub = itemp
         if (isub .eq. 1) then
            call exttop (lukeys,ludisp,numext,exsymb,ilatt,isub)
         endif
      endif
      return
      end

c-----------------------------------------------------------------------

      subroutine chaaij (lukeys,ludisp,iomv,isym,a1,a2,a3,a4,a5,a6)

      character answer*1

      write(ludisp,'(a)') ' '
      write(ludisp,'(a,$)')' Do you have direct cell parameters ?  Y/N '
      read (lukeys,'(a)') answer
      if (answer .eq. 'N' .or. answer .eq. 'n') then
         iomv = 0
      elseif (answer .eq. 'Y' .or. answer .eq. 'y') then
         iomv = 1
      else
         return
      endif

      write (ludisp,'(a)') ' '
      if (isym .eq. 1) then
         if (iomv .eq. 1) then
            write (ludisp,'(a,$)') ' Give A : '
         else
            write (ludisp,'(a)') ' SSQ = (H*H+K*K+L*L)*A11'
            write (ludisp,'(a,$)') ' Give A11 : '
         endif
         read (lukeys,*) a1
         a2 = 0.0
         a3 = 0.0
         a4 = 0.0
         a5 = 0.0
         a6 = 0.0

      elseif (isym .eq. 2) then
         if (iomv .eq. 1) then
            write (ludisp,'(a,$)') ' Give A and C : '
         else
            write (ludisp,'(a)') ' SSQ = (H*H+K*K)*A11 + L*L*A33'
            write (ludisp,'(a,$)') ' Give A11 and A33 : '
         endif
         read (lukeys,*) a1,a2
         a3 = 0.0
         a4 = 0.0
         a5 = 0.0
         a6 = 0.0

      elseif (isym .eq. 3) then
         if (iomv .eq. 1) then
            write (ludisp,'(a,$)') ' Give A, B and C : '	
         else
            write (ludisp,'(a)') ' SSQ = H*H*A11 + K*K*A22 + L*L*A33'
            write (ludisp,'(a,$)') ' Give A11, A22 and A33 : '
         endif
         read (lukeys,*) a1,a2,a3
         a4 = 0.0
         a5 = 0.0
         a6 = 0.0

      elseif (isym .eq. 4) then
         if (iomv .eq. 1) then
            write (ludisp,'(a,$)') ' Give A and C : '
         else
            write (ludisp,'(a)') ' SSQ = (H*H+H*K+K*K)*A11 + L*L*A33'
            write (ludisp,'(a,$)') ' Give A11 and A33 : '
         endif
         read (lukeys,*) a1,a2
         a3 = 0.0
         a4 = 0.0
         a5 = 0.0
         a6 = 0.0

      elseif (isym .eq. 5) then
         if (iomv .eq. 1) then
            write (ludisp,'(a,$)') ' Give A, B, C and Beta : '
         else
            write (ludisp,'(a)')
     >      ' SSQ = H*H*A11 + K*K*A22 + L*L*A33 + H*L*A13'
            write (ludisp,'(a,$)') ' Give A11,A22,A33 and A13 : '
         endif
         read (lukeys,*) a1,a2,a3,a4
         a5 = 0.0
         a6 = 0.0

      elseif (isym .eq. 6) then
         if (iomv .eq. 1) then
            write (ludisp,'(a)')
     >      ' Give A, B, C, Alfa, Beta and Gamma : '
         else
            write (ludisp,'(a,a)') 
     >      ' SSQ = H*H*A11 + K*K*A22 + L*L*A33 +',
     >      '       H*K*A12 + H*L*A13 + K*L*A23'
            write (ludisp,'(a,$)') ' Give A11,A22,A33,A12,A13 and A33 :'
         endif
         read (lukeys,*) a1,a2,a3,a4,a5,a6

      endif

      return
      end

c-----------------------------------------------------------------------

      subroutine pirssq (lukeys,ludisp,ssqtab,rint,iantal)
c
c --- This routine enables the user to change some SSQ-values.
c
      dimension ssqtab(1),rint(1)
      character answer*1
c
   11 format (' ',a)
   12 format (' ',a,$)
c
 4711 continue
      itime = 0
      call ersscr (ludisp)
      write(ludisp,'(a)') ' What do you want to do ?'
      write(ludisp,'(a)') ' -----------------------------------------'
      write(ludisp,'(a)') ' 1) Insert a new element in the SSQ-array.'
      write(ludisp,'(a)') ' 2) Delete one element in the SSQ-array.'
      write(ludisp,'(a)') ' 3) Look at the SSQ-array.'
      write(ludisp,'(a)') ' 4) Sort the SSQ-array in increasing order.'
      write(ludisp,'(a)') ' 5) Exit from this routine.'
      write(ludisp,'(a)') ' '
 4712 continue
      write (ludisp,'(a,$)') ' Type your answer here ->'
      read (lukeys,'(a)') answer
 
      if (answer .eq. '1') then
         write (ludisp,11) ' '         
         WRITE (ludisp,12) ' Give new SSQ-element and Intensity ->'         
         READ (lukeys,*,ERR=101) STEMP,RITEMP
         IF (IANTAL .LT. 200) THEN
            N = IANTAL+1
            SSQTAB(N) = STEMP
            RINT(N) = RITEMP
            IANTAL = N
         ELSE
            WRITE (ludisp,11) ' '
            WRITE (ludisp,11) ' No space for more elements...'
            WRITE (ludisp,11) ' '

         ENDIF
      ELSEIF (ANSWER .EQ. '2') THEN
         WRITE (ludisp,11) ' '
         WRITE (ludisp,12) ' What number ?'
         READ (lukeys,*,ERR=101) NUMBER
         IF (NUMBER .GT. IANTAL) THEN
            WRITE (ludisp,11) ' '
            WRITE (ludisp,11) ' Too large number...'
            WRITE (ludisp,11) ' '

         ELSEIF (NUMBER .LT. 1) THEN
            WRITE (ludisp,11) ' '
            WRITE (ludisp,11) ' Too small number...'
            WRITE (ludisp,11) ' '

         ELSE
            IF (NUMBER .LT. IANTAL) THEN
               DO 201 I = NUMBER,IANTAL
                  SSQTAB(I) = SSQTAB(I+1)
                  RINT(I) = RINT(I+1)
  201          CONTINUE
            ENDIF            
            IANTAL = IANTAL-1
         ENDIF
      ELSEIF (ANSWER .EQ. '3') THEN
         WRITE (ludisp,11) ' '         
         DO 301 I = 1,IANTAL
            WRITE (ludisp,302) SSQTAB(I),RINT(I)
  302       FORMAT (' ',F15.6,F10.2)
  301    CONTINUE
         WRITE (ludisp,11) ' '         

      ELSEIF (ANSWER .EQ. '4') THEN
         CALL SFIXA (SSQTAB,RINT,IANTAL)

      ELSEIF (ANSWER .EQ. '5') THEN
         GOTO 4713

      ELSE
         ITIME = ITIME+1
         IF (ITIME .LE. 10) THEN
            GOTO 4712
         ENDIF
      ENDIF
C
      GOTO 4711
C
C --- Error routine...
C
  101 CONTINUE
      WRITE (ludisp,11) ' '
      WRITE (ludisp,11) ' Error during numeric read ...'
      WRITE (ludisp,11) ' '
      GOTO 4711
C
 4713 CONTINUE
      CALL SFIXA (SSQTAB,RINT,IANTAL)
C
      RETURN
      END


      SUBROUTINE HSORT2 (V,IX,ANTAL)
C-----------------------------------------------------------------------
C This routine sorts the numbers in the array V in increasing order
C with heap sort ...
C-----------------------------------------------------------------------
      INTEGER   ANTAL
      DIMENSION V(1),IX(1)
C
C --- Create the heap.
C
      CALL CRHEP2 (V,IX,ANTAL)
C
C --- Sort the heap.
C
      DO 100 I = ANTAL,2,-1
         CALL SWAPR (V(I),V(1))
         CALL SWAPI (IX(I),IX(1))
         IROT = 1
         CALL ADHEP2 (V,IX,IROT,I-1)
  100 CONTINUE
C
      RETURN
      END


      SUBROUTINE ADHEP2 (V,IX,ROT,N)
C-----------------------------------------------------------------------
C This routine adjusts the heap ...
C-----------------------------------------------------------------------
      INTEGER   ROT,N
      DIMENSION V(1),IX(1)
C
 4711 CONTINUE
C
      J = 2*ROT
      IF (J .LE. N) THEN
         IF (J .LT. N) THEN
            IF (V(J) .LT. V(J+1)) THEN
               J = J+1
            ENDIF
         ENDIF
         IF (V(ROT) .LT. V(J)) THEN
            CALL SWAPR (V(J),V(ROT))
            CALL SWAPI (IX(J),IX(ROT))
            ROT = J
            GOTO 4711
         ENDIF
      ENDIF
C
      RETURN
      END


      SUBROUTINE CRHEP2 (V,IX,ANTAL)
C-----------------------------------------------------------------------
C This routine creates the heap ...
C-----------------------------------------------------------------------
      INTEGER   ANTAL
      DIMENSION V(1),IX(1)
C
      DO 100 I = ANTAL/2,1,-1
         IROT = I
         CALL ADHEP2 (V,IX,IROT,ANTAL)
  100 CONTINUE
C
      RETURN
      END
 

      SUBROUTINE SFIXA (X,Y,N)
C-----------------------------------------------------------------------
C This routine sort the values in the array "X" in increasing order
C with the help of heap-sort.  The values in the array "Y" corresponds
C to the same x-values as they did before, ie. they are sorted in a
C similar way as X are ...
C
C The actual dimension of X and Y in this case are 200 so the 
C index array are given this dimension.
C-----------------------------------------------------------------------
      DIMENSION X(1),Y(1)
      INTEGER   IND(200)
      REAL      YTEMP(200)
C
C --- First set the N first elements in the index vector to the
C     corresponding ordinal numbers.
C
      do 200 i = 1,n
         ind(i) = i
  200 continue
c
c --- Sort the X-array.
c
      call hsort2 (x,ind,n)
c
c --- Rearrange the values in the Y-array according to the IND-array.
c
      do 100 i = 1,n
         ytemp(i) = y(ind(i))
  100 continue
      do 101 i = 1,n
         y(i) = ytemp(i)
  101 continue
c
      return
      end

c-----------------------------------------------------------------------

      subroutine swapi (ia,ib)
c
c --- This routine swaps the two integer variables IA and IB.
c
      itemp = ia
      ia    = ib
      ib    = itemp

      return
      end

c-----------------------------------------------------------------------

      subroutine swapr (ra,rb)
c
c --- This routine swaps the two real variables RA and RB.
c
      temp = ra
      ra   = rb
      rb   = temp

      return
      end

c-----------------------------------------------------------------------

      subroutine blkstr (c80,ifrom,ito)
c
c --- This routine sets the positions beginning at IFROM up to and 
c     including ITO in the character-string C80 to blanks.
c
      character c80*80

      i = ifrom
  100 continue
      if (i .le. ito) then
         c80(i:i) = ' '
         i = i+1
         goto 100
      endif

      return
      end

c-----------------------------------------------------------------------

      subroutine ersscr (ludisp)
      write(ludisp,'(a)') ' '//char(27)//'[2J'
      return
      end

c-----------------------------------------------------------------------

      integer function istart (cline)
c
c --- This function returns the first non - blank position in "CLINE"
c
      character cline*80

      do 10 i = 1,80
         if (cline(i:i) .ne. ' ') then
            goto 20
         endif
   10 continue

   20 istart = i

      return
      end
 
c-----------------------------------------------------------------------
 
      integer function last (cline)
      character cline*80
      do 10 i = 80,1,-1
         if (cline(i:i) .ne. ' ') then
            last = i
            return
         endif
   10 continue
      last = 0
      return
      end

c-----------------------------------------------------------------------

      subroutine prdnew (lukeys,ludisp,iin,titstr,cpar,
     >                   ssqtab,rint,iantal,
     >                   dt,wt,ip,wave,ihm,ikm,ilm,isym,inr,isub,
     >                   exsymb,numext,ilatt,
     >                   a1,a2,a3,a4,a5,a6,iomv,
     >                   iht,ikt,ilt,ihi,iki,ili)
c
c --- This is the routine that enables the user to read the parameters
c     to pirum from a file that contains all vital information.
c
      integer    exsymb(50,5),numext,ilatt
      character  cpar*80, fname*80, titstr*80, st1*80, st2*80
      logical    filflg, titflg
c
      dimension  ssqtab(200),rint(200),
     >           iht(400),ikt(400),ilt(400),t1(200),tagut(200),
     >           ihi(400),iki(400),ili(400)
c
c --- get input file name ...
c
      if (cpar .ne. ' ') then
         fname = cpar
      else 
          if(IP .eq. 2) then
          write (ludisp,'(a)') ' IP=2 indexed data '
          endif
         write (ludisp,'(a,$)') ' Input file name ..... : '
         read (lukeys,'(a)') fname
         if (fname .eq. ' ') then
            write (ludisp,'(a)') ' Blnak filenames are not allowed ...'
            return
         endif
      endif
      open (unit=iin,file=fname,status='old',err=50)
      goto 60
c
c --- error section for the file opening.
c
   50 continue
      ler00 = istart (fname)
      ler01 = last   (fname)
      if (ler01 .lt. ler00) then
         ler01 = ler00
      endif
      write (ludisp,'(a)') ' '
      write (ludisp,'(a,a,a)')
     > ' error opening the file /',fname(ler00:ler01),'/'
      write (ludisp,'(a)') ' '
      return
c
c --- Everything OK , go on with the reading from the input file.
c
   60 continue
      if (ip .eq. 2) then
          i=1
          iantal=0
          ri_max=0.0
   62     continue
          read(iin,*,end=221) iht(i),ikt(i),ilt(i),ssqtab(i),rint(i)
          ihi(i)=iht(i)
          iki(i)=ikt(i)
          ili(i)=ilt(i)
           if(rint(i) .gt. ri_max) then
           ri_max=rint(i)
           endif
          i=i+1
          iantal=iantal+1
          goto 62
  221  if(ri_max .gt. 0.0) then
       i=0
       do while(i .lt. iantal)
       i=i+1
       rint(i)=(100.0*rint(i))/ri_max
       enddo
       endif
      goto 100
      endif
61    read(iin,'(a)') titstr
c
c --- Read the cards with sin square theta and intensity.
c     read one line, until EOF or a blank line is detected.
c
      i = 0
  170 continue
      read (iin,'(a)',end=180) st1
      if (st1 .eq. ' ') then
         goto 180
      else
         call strip(st1)
         n = nextbl(st1)
         st2 = st1(n:80)
         st1(n:80) = ' '
C         read (st1,'(f20.0)',iostat=istat) sqtemp
         call get_real(st1,istat,sqtemp)
         if (istat .ne. 0) then
            write(ludisp,'(a)') ' Error during internal read.'
            goto 170
         endif
         if (sqtemp .gt. 0.0) then
            i = i+1
            ssqtab(i) = sqtemp
         else
            goto 180
         endif            
         if (st2 .eq. ' ') then
            rint(i) = 0.0
         else
            if (st2 .ne. ' ') then
               call strip(st2)
               read (st2,'(f20.0)') rint(i)
               if (istat .ne. 0) then
                  write(ludisp,'(a)') ' Error during internal read.'
                  goto 170
               endif
            else
               rint(i) = 0.0
            endif
         endif
         goto 170
      endif
c
c --- Read some more control parameters...
c     OBS !!! Free format reading.
c
  180 continue
      iantal = i
c
c --- Maybe a sorting of the sine square data ?
c
      call sfixa (ssqtab,rint,iantal)
      read(iin,*) dt,wt,rip,wave,ihm,ikm,ilm,isym,inr,isub
      read(iin,*) a1,a2,a3,a4,a5,a6,iomv
      ip=rip
      if (isub .eq. 1) then
         read(iin,*,iostat=istat) numext,ilatt
         if (istat .ne. 0) then
            write(ludisp,'(a)')
     >      ' Error reading the number of reflexion conditions.'
            goto 100
         endif
         do 90 i = 1,numext
            read(iin,*) (exsymb(i,j),j=1,5)
   90    continue
      endif

  100 continue
      close (iin)

      return
      end
 
c-----------------------------------------------------------------------
 
      SUBROUTINE PW50(A11,A22,A33,A13,A12,A23,ISYM,IHM,IKM,ILM,
     >                IH,IK,IL,SSQT,IA)
C-----------------------------------------------------------------------
C     SE P-E 850
C-----------------------------------------------------------------------
      IA=1
      GO TO(10,90,160,230,300,390),ISYM
   10 IF(IH-IHM) 20,30,30
   20 IH=IH+1
      GO TO 70
   30 IF(IK-IKM) 40,50,50
   40 IK=IK+1
      IH=IK
      GO TO 70
   50 IF(IL-ILM) 60,80,80
   60 IL=IL+1
      IK=IL
      IH=IK
   70 SSQT=FLOAT(IH*IH+IK*IK+IL*IL)*A11
      RETURN
   80 IA=0
      RETURN
   90 IF(IH-IHM) 100,110,110
  100 IH=IH+1
      GO TO 150
  110 IF(IK-IKM) 120,130,130
  120 IK=IK+1
      IH=IK
      GO TO 150
  130 IF(IL-ILM) 140,80,80
  140 IL=IL+1
      IH=0
      IK=0
  150 SSQT=FLOAT(IH*IH+IK*IK)*A11+FLOAT(IL*IL)*A22
      RETURN
  160 IF(IH-IHM) 170,180,180
  170 IH=IH+1
      GO TO 220
  180 IH=0
      IF(IK-IKM) 190,200,200
  190 IK=IK+1
      GO TO 220
  200 IF(IL-ILM) 210,80,80
  210 IK=0
      IL=IL+1
  220 SSQT=FLOAT(IH*IH)*A11+FLOAT(IK*IK)*A22+FLOAT(IL*IL)*A33
      RETURN
C
C     CHANGE S50/46,47,48,49,50,54
C
  230 IF(IH-IHM) 240,250,250
  240 IH=IH+1
      GO TO 290
  250 IF(IK-IKM) 260,270,270
  260 IK=IK+1
      IH=IK
      GO TO 290
  270 IF(IL-ILM) 280,80,80
  280 IH=0
      IK=0
      IL=IL+1
  290 SSQT=FLOAT(IH*IH+IH*IK+IK*IK)*A11+FLOAT(IL*IL)*A22
      RETURN
  300 IF(IH*IL)320,320,310
  310 IH=-IH
      GO TO 380
  320 IF(IABS(IH)-IHM) 330,340,340
  330 IH=IABS(IH)+1
      GO TO 380
  340 IH=0
      IF(IK-IKM)350,360,360
  350 IK=IK+1
      GO TO 380
  360 IF(IL-ILM) 370,80,80
  370 IK=0
      IL=IL+1
  380 SSQT=FLOAT(IH*IH)*A11+FLOAT(IK*IK)*A22+FLOAT(IL*IL)*A33+
     >     FLOAT(IH*IL)*A13
      RETURN
  390 IF(IABS(IK)+IL) 400,420,400
  400 IF(IH) 420,420,410
  410 IH=-IH
      GO TO 510
  420 IF(IABS(IH)-IHM) 430,440,440
  430 IH=IABS(IH)+1
      GO TO 510
  440 IH=0
      IF(IL) 470,470,450
  450 IF(IK) 470,470,460
  460 IK=-IK
      GO TO 510
  470 IF(IABS(IK)-IKM) 480,490,490
  480 IK=IABS(IK)+1
      GO TO 510
  490 IK=0
      IF(IL-ILM) 500,80,80
  500 IL=IL+1
  510 SSQT=FLOAT(IH*IH)*A11+FLOAT(IK*IK)*A22+FLOAT(IL*IL)*A33+
     >     FLOAT(IH*IL)*A13+FLOAT(IH*IK)*A12+FLOAT(IK*IL)*A23
      RETURN
      END


      SUBROUTINE PW51(SSQT,T1,N,T11,DT,SSQTO,IA)
C-----------------------------------------------------------------------
C     SE P-E 852
C-----------------------------------------------------------------------
      DIMENSION T1(1),T11(11)
      HA=SQRT(SSQT)
      HB=SQRT(1.-SSQT)
      TTH=114.59156*ATAN(HA/HB)
      DO 40 I=1,N
      IF(ABS(TTH-T11(I))-DT) 50,50,40
C      IF(SSQT-SSQTL) 10,20,20
C   10 D=DT
C      GO TO 30
C   20 D=D2
C   30 DO 40 I=1,N
C      IF(ABS(SSQT-T1(I))-D) 50,50,40
   40 CONTINUE
      IA=0
      RETURN
   50 IA=1
      SSQTO=T1(I)
      RETURN
      END


      SUBROUTINE PW52(N,NEKV,ISYM,V2,WAVE,AMAT,S,A,B,C,AL,BE,GA,
     >                DA,DB,DC,DAL,DBE,DGA,VOLM)
C-----------------------------------------------------------------------
      DIMENSION AMAT(6,6),S(6,6),F(6)
      SIG=SQRT(V2/(FLOAT(NEKV-N)))
      SIG=(4.0*SIG)/(WAVE**2)
      DO 20 L=1,N
      F(L)=0.0
      DO 10 J=1,N
      DO 10 K=1,N
   10 F(L)=F(L)+AMAT(J,K)*S(L,J)*S(L,K)
   20 F(L)=SIG*SQRT(F(L))
      DB=0.0
      DC=0.0
      DAL=0.0
      DBE=0.0
      DGA=0.0
      RAD=0.0174533
      GRAD=57.2958
      DA=F(1)
      GO TO(30,40,50,60,70,80),ISYM
   30 B=A
      C=A
      DB=DA
      DC=DA
      GO TO 150
   40 B=A
      DB=DA
      DC=F(2)
      GO TO 150
   50 DB=F(2)
      DC=F(3)
      GO TO 150
   60 DB=DA
      B=A
      DC=F(2)
      GA=120.0
      GO TO 180
   70 DB=F(2)
      DC=F(3)
      BE=GRAD*BE
      DBE=GRAD*F(4)
      GO TO 190
   80 DB=F(2)
      DC=F(3)
      AL=GRAD*AL
      IF(AL) 90,100,100
   90 AL=AL+180.0
  100 DAL=GRAD*F(4)
      BE=GRAD*BE
      IF(BE) 110,120,120
  110 BE=BE+180.0
  120 DBE=GRAD*F(5)
      GA=GRAD*GA
      IF(GA) 130,140,140
  130 GA=GA+180.0
  140 DGA=GRAD*F(6)
      GO TO 200
  150 VOLM=A*B*C
      BE=90.0
  160 GA=90.0
  170 AL=90.0
      GO TO 210
  180 VOLM=0.8660254*A*B*C
      BE=90.0
      GO TO 170
  190 VOLM=A*B*C*SIN(RAD*BE)
      GO TO 160
  200 CA=COS(RAD*AL)
      CB=COS(RAD*BE)
      CC=COS(RAD*GA)
      VOLM=A*B*C*SQRT(1.0-CA**2-CB**2-CC**2+2.0*CA*CB*CC)
  210 RETURN
      END


      SUBROUTINE PW53(SSQTO,T2,T22,WG,N2,IHT,IKT,ILT,IQE,IH,IK,IL,ISYM,
     >                WT)
C-----------------------------------------------------------------------
C     SE P-E 853 OBS. N2 START=0
C-----------------------------------------------------------------------
      DIMENSION T2(1),IHT(1),IKT(1),ILT(1),IQE(1),T22(1),WG(1)
      I=1
   10 IF(I-N2) 30,30,20
   20 T2(I)=SSQTO
      HA=SQRT(SSQTO)
      HB=SQRT(1.-SSQTO)
      T22(I)=114.59156*ATAN(HA/HB)
       IF(WT .EQ. 1.) THEN
         WG(I)=1.
         ELSE 
       WG(I)=1./((2.*HA*HB)**2)
       ENDIF
      IHT(I)=IH
      IKT(I)=IK
      ILT(I)=IL
      IQE(I)=1
      GO TO 160
   30 IF(SSQTO-T2(I)) 40,50,40
   40 I=I+1
      GO TO 10
   50 GO TO (60,80,100,120,130,140),ISYM
   60 IF((IH**2+IK**2+IL**2)-(IHT(I)**2+IKT(I)**2+ILT(I)**2))70,170,70
   70 IQE(I)=0
      RETURN
   80 IF((IH**2+IK**2)-(IHT(I)**2+IKT(I)**2)) 70,90,70
   90 IF(IL**2-ILT(I)**2) 70,170,70
  100 IF(IH**2-IHT(I)**2) 70,110,70
  110 IF(IK**2-IKT(I)**2) 70,90,70
  120 IF((IH**2+IH*IK+IK**2)-(IHT(I)**2+IHT(I)*IKT(I)+IKT(I)**2))
     >  70,90,70
  130 IF(IH*IL-IHT(I)*ILT(I)) 70,100,70
  140 IF(IH*IK-IHT(I)*IKT(I)) 70,150,70
  150 IF(IK*IL-IKT(I)*ILT(I)) 70,130,70
  160 SSQTO=-SSQTO
  170 RETURN
      END


      SUBROUTINE PW54(A,N,B)
C-----------------------------------------------------------------------
C     M=0 GER AINV,M=1 DESSUTOM X I B-VEKTORN
C-----------------------------------------------------------------------
      DIMENSION IPIV(6),A(6,6),B(6,1),INDEX(6,2),PIVOT(6)
      EQUIVALENCE (IROW,JROW),(ICOL,JCOL),(AMAX,T,SWAP)
      M=1
      DET=1.0
      DO 10 J=1,N
   10 IPIV(J)=0
      DO 200 I=1,N
      AMAX=0.0
      DO 60 J=1,N
      IF(IPIV(J)-1) 20,60,20
   20 DO 50 K=1,N
      IF(IPIV(K)-1) 30,50,240
   30 IF(ABS(AMAX)-ABS(A(J,K))) 40,50,50
   40 IROW=J
      ICOL=K
      AMAX=A(J,K)
   50 CONTINUE
   60 CONTINUE
      IPIV(ICOL)=IPIV(ICOL)+1
      IF(IROW-ICOL) 70,110,70
   70 DET=-DET
      DO 80 L=1,N
      SWAP=A(IROW,L)
      A(IROW,L)=A(ICOL,L)
   80 A(ICOL,L)=SWAP
      IF(M)110,110,90
   90 DO 100 L=1,M
      SWAP=B(IROW,L)
      B(IROW,L)=B(ICOL,L)
  100 B(ICOL,L)=SWAP
  110 INDEX(I,1)=IROW
      INDEX(I,2)=ICOL
      PIVOT(I)=A(ICOL,ICOL)
      DET=DET*PIVOT(I)
      A(ICOL,ICOL)=1.0
      DO 120 L=1,N
  120 A(ICOL,L)=A(ICOL,L)/PIVOT(I)
      IF(M) 150,150,130
  130 DO 140 L=1,M
  140 B(ICOL,L)=B(ICOL,L)/PIVOT(I)
  150 DO 200 L1=1,N
      IF(L1-ICOL) 160,200,160
  160 T=A(L1,ICOL)
      A(L1,ICOL)=0.0
      DO 170 L=1,N
  170 A(L1,L)=A(L1,L)-A(ICOL,L)*T
      IF(M) 200,200,180
  180 DO 190 L=1,M
  190 B(L1,L)=B(L1,L)-B(ICOL,L)*T
  200 CONTINUE
      DO 230 I=1,N
      L=N+1-I
      IF(INDEX(L,1)-INDEX(L,2)) 210,230,210
  210 JROW=INDEX(L,1)
      JCOL=INDEX(L,2)
      DO 220 K=1,N
      SWAP=A(K,JROW)
      A(K,JROW)=A(K,JCOL)
      A(K,JCOL)=SWAP
  220 CONTINUE
  230 CONTINUE
  240 RETURN
      END

      SUBROUTINE PW56(T1,T2,IHT,IKT,ILT,N1,N2,IOUT,DT,WT,IP,WAVE,
     >           TAGUT,N20,N30,NFI,NT20,NT30,SQ20,SQ30,SQFI,M20,NOIND,
     >           T11,T22,IOUP)
C-----------------------------------------------------------------------
      DIMENSION T1(1),T2(1),IHT(1),IKT(1),ILT(1),TAGUT(1),T11(1),T22(1)
      isense=0
       if (iout .lt. 0) then
       isense=1
       iout=-iout
       write(iout,500) DT
  500   format(' ',' All calculated lines where...',/,
     >  '  ABS(2theta-obs  minus 2theta-calc) < ',F5.3,' deg.',/,
     >  '  are listed below. ')
       endif
      NOIND=0
      NWIND=0
      M20=0
      IF(SQ20.EQ.0.) GO TO 10
      SAK=SQ20
      GO TO 20
   10 SAK=SQFI
   20 CONTINUE
      DO 50 J1=1,N2
      ISH=IHT(J1)
      ISK=IKT(J1)
      ISL=ILT(J1)
      TS=T2(J1)
      TSS=T22(J1)
      IMH=ISH
      IMK=ISK
      IML=ISL
      TM=TS
      TMM=TSS
      DO 40 J2=J1,N2
      IF(TM-T2(J2)) 40,30,30
   30 TM=T2(J2)
      TMM=T22(J2)
      IMH=IHT(J2)
      IMK=IKT(J2)
      IML=ILT(J2)
      J3=J2
   40 CONTINUE
      IHT(J1)=IMH
      IKT(J1)=IMK
      ILT(J1)=IML
      T2(J1)=TM
      T22(J1)=TMM
      IHT(J3)=ISH
      IKT(J3)=ISK
      ILT(J3)=ISL
      T22(J3)=TSS
   50 T2(J3)=TS
      DSUM=0.0
      DSUMA=0.0
      DSUMB=0.0
      DSUMC=0.0
      TSUMA=0.0
      TSUMB=0.0
      TSUMC=0.0
      NTALA=0
      NTALB=0
      NTALC=0
      GRAD=114.59156
      WRITE(IOUT,60)
   60 FORMAT(' ','   H   K   L SST-OBS  SST-CALC   DELTA   2TH-OBS',
     >           ' 2TH-CALC D-OBS   FREE PARAM.')
      J=1
      DO 280 I=1,N2
   70 IF(J-N1) 80,80,260
   80 IF(I-N2) 90,100,100
   90 IF(ABS(T11(J)-T22(I))-ABS(T11(J)-T22(I+1))) 100,100,260  
C   90 IF(ABS(T1(J)-T2(I))-ABS(T1(J)-T2(I+1))) 100,100,260
  100 IF(ABS(T11(J)-T22(I))-DT) 180,180,130  
C  100 IF(T1(J)-SSQTL) 110,110,120
C  110 IF(ABS(T1(J)-T2(I))-D1) 180,180,130
C  120 IF(ABS(T1(J)-T2(I))-D2) 180,180,130
C  130 IF(T1(J)-T2(I)) 140,140,260
  130 IF(T11(J)-T22(I)) 140,140,260
  140 HA=SQRT(T1(J))
      DOBS=WAVE/(2.*HA)
      HD=SQRT(1.-T1(J))
      THO=GRAD*ATAN(HA/HD)
      NOIND=NOIND+1
      WRITE(IOUT,150) T1(J),THO,DOBS,TAGUT(J)
  150 FORMAT(' ',12X,F9.6,19X,F8.3,8X,F8.4,2X,F10.2)
      IF(T1(J)-SAK) 160,160,170
  160 NWIND=NWIND+1
  170 J=J+1
      GO TO 70
  180 HA=SQRT(T1(J))
      HB=SQRT(T2(I))
      DOBS=WAVE/(2.*HA)
      DCALC=WAVE/(2.*HB)
      DELTA=T1(J)-T2(I)
      HD=SQRT(1.-T2(I))
      HE=SQRT(1.-T1(J))
      THO=ATAN(HA/HE)*GRAD
      THC=ATAN(HB/HD)*GRAD
      DELTB=THO-THC
C      WRITE(IOUT,190) IHT(I),IKT(I),ILT(I),T1(J),T2(I),DELTA,THO,THC,
      WRITE(IOUT,190) IHT(I),IKT(I),ILT(I),T1(J),T2(I),DELTB,THO,THC,
     >                DOBS,TAGUT(J)
  190 FORMAT(' ',3I4,2F9.6,F10.3,2F8.3,F8.4,2X,F10.2)
      IF(IP .EQ. 1) THEN
      WRITE(IOUP,201) IHT(I),IKT(I),ILT(I),T1(J),TAGUT(J)
  201 FORMAT(3I5,F10.6,F10.2)
      ENDIF
      IF(SQ20-T1(J)) 210,200,200
  200 DSUMA=DSUMA+ABS(DELTA)
      TSUMA=TSUMA+ABS(DELTB)
      NTALA=NTALA+1
  210 IF(SQ30-T1(J)) 230,220,220
  220 DSUMB=DSUMB+ABS(DELTA)
      TSUMB=TSUMB+ABS(DELTB)
      NTALB=NTALB+1
  230 IF(SQFI-T1(J)) 250,240,240
  240 DSUMC=DSUMC+ABS(DELTA)
      TSUMC=TSUMC+ABS(DELTB)
      NTALC=NTALC+1
  250 J=J+1
      GO TO 280
  260 HC=SQRT(T2(I))
      HD=SQRT(1.-T2(I))
      THC=GRAD*ATAN(HC/HD)
      WRITE(IOUT,270) IHT(I),IKT(I),ILT(I),T2(I),THC
  270 FORMAT(' ',3I4,9X,F9.6,18X,F8.3)
  280 CONTINUE
  290 IF(J-N1) 300,300,330
  300 HA=SQRT(T1(J))
      HD=SQRT(1.-T1(J))
      DOBS=WAVE/(2.*HA)
      NOIND=NOIND+1
      THO=GRAD*ATAN(HA/HD)
      WRITE(IOUT,150) T1(J),THO,DOBS,TAGUT(J)
C     omkastat tidigare....
      IF(T1(J)-SAK) 310,310,320
  310 NWIND=NWIND+1
  320 J=J+1
      GO TO 290
  330 CONTINUE
      WRITE(IOUT,331)
  331 FORMAT(' ')
      WRITE(IOUT,340) N1
  340 FORMAT(' Number of obs. lines  =',I5)
      WRITE(IOUT,350) N2
  350 FORMAT(' Number of calc. lines =',I5)
      IF(NTALA) 380,380,360
  360 TMEDA=TSUMA/NTALA
      DMEDA=DSUMA/NTALA
      MER=SQ20/(2.0*DMEDA*N20)+0.5
      M20=MER
      F20=20.0/(TMEDA*N20)+0.5
      NDEF=20
      if(isense .eq. 0) then
      WRITE(IOUT,331)
      WRITE(IOUT,370) NDEF,MER,DMEDA,NDEF,F20,TMEDA,N20
  370 FORMAT(' M(',I3,') =',I4,'  AV.EPS.=',F10.7,/,
     >       ' F(',I3,') =',F5.0,'(',F8.6,',',I5,')')
      endif
  380 IF(NTALB) 400,400,390
  390 TMEDB=TSUMB/NTALB
      DMEDB=DSUMB/NTALB
      MER=SQ30/(2.0*DMEDB*N30)+0.5
      F30=30.0/(TMEDB*N30)+0.5
      NDEF=30
      if(isense .eq. 0) then
      WRITE(IOUT,331)
      WRITE(IOUT,370) NDEF,MER,DMEDB,NDEF,F30,TMEDB,N30
      endif
  400 IF(NTALC) 420,420,410
  410 DMEDC=DSUMC/NTALC
      TMEDC=TSUMC/NTALC
      FN1=N1/(TMEDC*NFI)+0.5
      MER=SQFI/(2.0*DMEDC*NFI)+0.5
      IF(M20.EQ.0) M20=MER
      if(isense .eq. 0) then
      WRITE(IOUT,331)
      WRITE(IOUT,370) N1,MER,DMEDC,N1,FN1,TMEDC,NFI
      endif
  420 CONTINUE
      WRITE(IOUT,331)
C
C --- Write references for the M- and F-indexes.
C
      if(isense .eq. 0) then
       if(wt .eq. 1.) then
         write(iout,'(a)') ' UNIT weights used '
         else
         write(iout,'(a)') ' HESS weights used '
       endif
      WRITE(IOUT,331)
      write(iout,430)
  430 format(' M  cf. J.Appl.Cryst.  1 (1968) 108 ',/,
     >       ' F  cf. J.Appl.Cryst. 12 (1979)  60 ')
      endif
C
C --- Write the number of indexed lines (in a nice format) ...
C
      write(iout,331)
      if (noind .lt. 10) then
         write(iout,441) noind
      elseif (noind .lt. 100) then
         write(iout,442) noind
      elseif (noind .lt. 1000) then
         write(iout,443) noind
      elseif (noind .lt. 10000) then
         write(iout,444) noind
      else
         write(iout,445) noind
      endif
  441 format(' ',I1,' lines are unindexed. ')
  442 format(' ',I2,' lines are unindexed. ')
  443 format(' ',I3,' lines are unindexed. ')
  444 format(' ',I4,' lines are unindexed. ')
  445 format(' ',I5,' lines are unindexed. ')
C
      NOIND=NWIND
      RETURN
      END
      SUBROUTINE PW57(T1,T2,IHT,IKT,ILT,N1,N2,IOUT,DT,WT,IP,WAVE,
     >           TAGUT,N20,N30,NFI,NT20,NT30,SQ20,SQ30,SQFI,M20,NOIND,
     >           T11,T22,IHI,IKI,ILI,IOUP,A11,A22,A33,A13,A12,A23)
C-----------------------------------------------------------------------
      DIMENSION T1(1),T2(1),IHT(1),IKT(1),ILT(1),TAGUT(1),T11(1),T22(1),
     >          IHI(1),IKI(1),ILI(1)
      DIMENSION ITAG(200)
      do 237 i=1,n1
      itag(i)=tagut(i)
  237 continue
      WRITE(IOUT,331)
      write(iout,195)
  195 format(' No cycles as (hkl)s are fixed ',/,
     >' Any input of cell parameters irrelevant ',/,
     >' Extinction conditions will affect figures of merits ')
      write(iout,196) dt
  196 format(' The diff. parameter=',F5.3,
     >       ' has no influence on cell parameters ')
      WRITE(IOUT,331)
      NOIND=0
      NWIND=0
      NR=1
      M20=0
      IF(SQ20.EQ.0.) GO TO 10
      SAK=SQ20
      GO TO 20
   10 SAK=SQFI
   20 CONTINUE
      DO 50 J1=1,N2
      ISH=IHT(J1)
      ISK=IKT(J1)
      ISL=ILT(J1)
      TS=T2(J1)
      TSS=T22(J1)
      IMH=ISH
      IMK=ISK
      IML=ISL
      TM=TS
      TMM=TSS
      DO 40 J2=J1,N2
      IF(TM-T2(J2)) 40,30,30
   30 TM=T2(J2)
      TMM=T22(J2)
      IMH=IHT(J2)
      IMK=IKT(J2)
      IML=ILT(J2)
      J3=J2
   40 CONTINUE
      IHT(J1)=IMH
      IKT(J1)=IMK
      ILT(J1)=IML
      T2(J1)=TM
      T22(J1)=TMM
      IHT(J3)=ISH
      IKT(J3)=ISK
      ILT(J3)=ISL
      T22(J3)=TSS
   50 T2(J3)=TS
      DSUM=0.0
      DSUMA=0.0
      DSUMB=0.0
      DSUMC=0.0
      TSUMA=0.0
      TSUMB=0.0
      TSUMC=0.0
      NTALA=0
      NTALB=0
      NTALC=0
      GRAD=114.59156
      WRITE(IOUT,61)
      WRITE(IOUT,60)
   61 FORMAT(' ','                                              '
     >           '  Calculated')
   60 FORMAT(' ','   H   K   L  2Th-obs 2Th-calc    D-obs  I-obs'
     >           '   H   K   L Delta(2Th)')
C   60 FORMAT(' ','   H   K   L SST-OBS  SST-CALC   DELTA   2TH-OBS',
C     >           ' 2TH-CALC D-OBS   FREE PARAM.')
      J=1
      DO 280 I=1,N2
   70 IF(J-N1) 80,80,260
   80 IF(I-N2) 90,100,100
   90 IF(ABS(T11(J)-T22(I))-ABS(T11(J)-T22(I+1))) 100,100,260  
C   90 IF(ABS(T1(J)-T2(I))-ABS(T1(J)-T2(I+1))) 100,100,260
  100 IF(ABS(T11(J)-T22(I))-DT) 180,180,130  
C  100 IF(T1(J)-SSQTL) 110,110,120
C  110 IF(ABS(T1(J)-T2(I))-D1) 180,180,130
C  120 IF(ABS(T1(J)-T2(I))-D2) 180,180,130
C  130 IF(T1(J)-T2(I)) 140,140,260
  130 IF(T11(J)-T22(I)) 140,140,260
  140 HA=SQRT(T1(J))
      DOBS=WAVE/(2.*HA)
      HD=SQRT(1.-T1(J))
      THO=GRAD*ATAN(HA/HD)
      NOIND=NOIND+1
C      WRITE(IOUT,150) T1(J),THO,DOBS,TAGUT(J)
      lh=ihi(j)
      lk=iki(j)
      ll=ili(j)
      ssqtc=lh*lh*a11+lk*lk*a22+ll*ll*a33+lh*ll*a13+lh*lk*a12+lk*ll*a23
      ha=sqrt(ssqtc)
      hb=sqrt(1.-ssqtc)
      thcalc=atan(ha/hb)*grad      
      deltc=tho-thcalc
      if (dobs .lt. 2.501) then
      write(iout,150) lh,lk,ll,tho,thcalc,dobs,itag(j),     
     >                lh,lk,ll,deltc,dt
  150 FORMAT(' ',3I4,2F9.3,F9.4,I7,3I4,F11.3,' >'F5.3)
      else if(dobs .lt. 5.001) then
      write(iout,151) lh,lk,ll,tho,thcalc,dobs,itag(j),     
     >                lh,lk,ll,deltc,dt
  151 FORMAT(' ',3I4,2F9.3,F8.3,1X,i7,3I4,F11.3,' >'F5.3)
      else 
      write(iout,152) lh,lk,ll,tho,thcalc,dobs,itag(j),     
     >                lh,lk,ll,deltc,dt
  152 FORMAT(' ',3I4,2F9.3,F7.2,2X,I7,3I4,F11.3,' >'F5.3)
      endif
C      WRITE(IOUT,150) IHI(J),IKI(J),ILI(J),THO,DOBS,TAGUT(J)
C  150 FORMAT(' ',12X,F9.6,19X,F8.3,8X,F8.4,2X,F10.2)
      IF(T1(J)-SAK) 160,160,170
  160 NWIND=NWIND+1
  170 J=J+1
      GO TO 70
  180 HA=SQRT(T1(J))
      HB=SQRT(T2(I))
      DOBS=WAVE/(2.*HA)
      DCALC=WAVE/(2.*HB)
      DELTA=T1(J)-T2(I)
      HD=SQRT(1.-T2(I))
      HE=SQRT(1.-T1(J))
      THO=ATAN(HA/HE)*GRAD
      THC=ATAN(HB/HD)*GRAD
      DELTB=THO-THC
C      WRITE(IOUT,190) IHT(I),IKT(I),ILT(I),T1(J),T2(I),DELTA,THO,THC,
C      WRITE(IOUT,190) IHT(I),IKT(I),ILT(I),T1(J),T2(I),DELTB,THO,THC,
C     >                DOBS,TAGUT(J)
            if(ihi(j) .eq. iht(i) .and. 
     >         iki(j) .eq. ikt(i) .and.
     >         ili(j) .eq. ilt(i)) then
      if(dobs .lt. 2.501) then
      WRITE(IOUT,190)  IHI(J),IKI(J),ILI(J),THO,THC,DOBS,ITAG(J),
     >                 IHT(I),IKT(I),ILT(I),DELTB
  190 FORMAT(' ',3I4,2F9.3,F9.4,I7,3I4,F11.3)
      else if(dobs .lt. 5.001) then
      WRITE(IOUT,153)  IHI(J),IKI(J),ILI(J),THO,THC,DOBS,ITAG(J),
     >                 IHT(I),IKT(I),ILT(I),DELTB
  153 FORMAT(' ',3I4,2F9.3,F8.3,1X,I7,3I4,F11.3)     
      else
      WRITE(IOUT,154)  IHI(J),IKI(J),ILI(J),THO,THC,DOBS,ITAG(J),
     >                 IHT(I),IKT(I),ILT(I),DELTB
  154 FORMAT(' ',3I4,2F9.3,F7.2,2X,I7,3I4,F11.3)     
      endif
            else
            lh=ihi(j)
            lk=iki(j)
            ll=ili(j)
      ssqtc=lh*lh*a11+lk*lk*a22+ll*ll*a33+lh*ll*a13+lh*lk*a12+lk*ll*a23
            ha=sqrt(ssqtc)
            hb=sqrt(1.-ssqtc)
            thcalc=atan(ha/hb)*grad
      if(dobs .lt. 2.501) then
      WRITE(IOUT,191)  IHI(J),IKI(J),ILI(J),THO,THC,DOBS,ITAG(J),
     >                 IHT(I),IKT(I),ILT(I),DELTB
  191 FORMAT(' ',3I4,2F9.3,F9.4,I7,3I4,F11.3,' *')
      else if(dobs .lt. 5.001) then
      WRITE(IOUT,197)  IHI(J),IKI(J),ILI(J),THO,THC,DOBS,ITAG(J),
     >                 IHT(I),IKT(I),ILT(I),DELTB
  197 FORMAT(' ',3I4,2F9.3,F8.3,1X,I7,3I4,F11.3,' *')
      else
      WRITE(IOUT,198)  IHI(J),IKI(J),ILI(J),THO,THC,DOBS,ITAG(J),
     >                 IHT(I),IKT(I),ILT(I),DELTB
  198 FORMAT(' ',3I4,2F9.3,F7.2,2X,I7,3I4,F11.3,' *')
      endif
            deltc=tho-thcalc
      if(dobs .lt. 2.501) then
      write(iout,192) thcalc,dobs,lh,lk,ll,deltc
  192 format(' ',21X,F9.3,F9.4,7X,3I4,F11.3,' !')
      else if(dobs .lt. 5.001) then
      write(iout,231) thcalc,dobs,lh,lk,ll,deltc
  231 format(' ',21X,F9.3,F8.3,8X,3I4,F11.3,' !')
      else 
      write(iout,232) thcalc,dobs,lh,lk,ll,deltc
  232 format(' ',21X,F9.3,F9.4,7X,3I4,F11.3,' !')
      endif
            endif
C  190 FORMAT(' ',3I4,2F9.6,F10.3,2F8.3,F8.4,2X,F10.2)
      IF(SQ20-T1(J)) 210,200,200
  200 DSUMA=DSUMA+ABS(DELTA)
      TSUMA=TSUMA+ABS(DELTB)
      NTALA=NTALA+1
  210 IF(SQ30-T1(J)) 230,220,220
  220 DSUMB=DSUMB+ABS(DELTA)
      TSUMB=TSUMB+ABS(DELTB)
      NTALB=NTALB+1
  230 IF(SQFI-T1(J)) 250,240,240
  240 DSUMC=DSUMC+ABS(DELTA)
      TSUMC=TSUMC+ABS(DELTB)
      NTALC=NTALC+1
  250 J=J+1
      GO TO 280
  260 HC=SQRT(T2(I))
      HD=SQRT(1.-T2(I))
      THC=GRAD*ATAN(HC/HD)
C      WRITE(IOUT,270) IHT(I),IKT(I),ILT(I),T2(I),THC
C  270 FORMAT(' ',3I4,9X,F9.6,18X,F8.3)
  280 CONTINUE
  290 IF(J-N1) 300,300,330
  300 HA=SQRT(T1(J))
      HD=SQRT(1.-T1(J))
      DOBS=WAVE/(2.*HA)
      NOIND=NOIND+1
C      WRITE(IOUT,150) T1(J),THO,DOBS,TAGUT(J)
      THO=GRAD*ATAN(HA/HD)
      WRITE(IOUT,501) IHI(J),IKI(J),ILI(J),THO,DOBS,ITAG(J)
  501 FORMAT(' ',3I4,F9.3,9X,F9.4,I7)
      IF(T1(J)-SAK) 310,310,320
  310 NWIND=NWIND+1
  320 J=J+1
      GO TO 290
  330 CONTINUE
      WRITE(IOUT,331)
  331 FORMAT(' ')
C      WRITE(IOUT,340) N1
C  340 FORMAT(' Number of obs. lines  =',I5)
      WRITE(IOUT,350) DT,N2
C  350 FORMAT(' Number of calc. lines =',I5)
  350 format(' Number of calc. lines. within ',F5.3,
     >       ' deg. from obs. 2-Theta=',i5)
      IF(NTALA) 380,380,360
  360 TMEDA=TSUMA/NTALA
      DMEDA=DSUMA/NTALA
      MER=SQ20/(2.0*DMEDA*N20)+0.5
      M20=MER
      F20=20.0/(TMEDA*N20)+0.5
      NDEF=20
      WRITE(IOUT,331)
      WRITE(IOUT,370) NDEF,MER,DMEDA,NDEF,F20,TMEDA,N20
  370 FORMAT(' M(',I3,') =',I4,'  AV.EPS.=',F10.7,/,
     >       ' F(',I3,') =',F5.0,'(',F8.6,',',I5,')')
  380 IF(NTALB) 400,400,390
  390 TMEDB=TSUMB/NTALB
      DMEDB=DSUMB/NTALB
      MER=SQ30/(2.0*DMEDB*N30)+0.5
      F30=30.0/(TMEDB*N30)+0.5
      NDEF=30
      WRITE(IOUT,331)
      WRITE(IOUT,370) NDEF,MER,DMEDB,NDEF,F30,TMEDB,N30
  400 IF(NTALC) 420,420,410
  410 DMEDC=DSUMC/NTALC
      TMEDC=TSUMC/NTALC
      FN1=N1/(TMEDC*NFI)+0.5
      MER=SQFI/(2.0*DMEDC*NFI)+0.5
      IF(M20.EQ.0) M20=MER
      WRITE(IOUT,331)
      WRITE(IOUT,370) N1,MER,DMEDC,N1,FN1,TMEDC,NFI
  420 CONTINUE
      WRITE(IOUT,331)
C
C --- Write references for the M- and F-indexes.
C
      if(wt .eq. 1.) then
        write(iout,'(a)') ' UNIT weights used '
        else
        write(iout,'(a)') ' HESS weights used '
      endif
      WRITE(IOUT,331)
      write(iout,430)
  430 format(' M  cf. J.Appl.Cryst.  1 (1968) 108 ',/,
     >       ' F  cf. J.Appl.Cryst. 12 (1979)  60 ')
C
C --- Write the number of indexed lines (in a nice format) ...
C
      write(iout,331)
      write(iout,441) dt,noind
  441 format(' Number of differences >',F5.3,' deg. =',i4)
      NOIND=NWIND
      RETURN
      END

      SUBROUTINE PW61(A,B,C,AL,BE,GA,S)
C-----------------------------------------------------------------------
C     TRICLINIC SYM. PARTIAL DERIVATIVES OF DIRECT LATTICE PARAMETERS
C-----------------------------------------------------------------------
      DIMENSION S(6,6)
      S(1,1)=-0.5*A*A*A
      S(1,2)=-0.5*A*B*B*COS(GA)**2
      S(1,3)=-0.5*A*C*C*COS(BE)**2
      S(1,4)=-0.5*A*B*C*COS(BE)*COS(GA)
      S(1,5)=-0.5*A*A*C*COS(BE)
      S(1,6)=-0.5*A*A*B*COS(GA)
      S(2,1)=-0.5*B*A*A*COS(GA)**2
      S(2,2)=-0.5*B*B*B
      S(2,3)=-0.5*B*C*C*COS(AL)**2
      S(2,4)=-0.5*B*B*C*COS(AL)
      S(2,5)=-0.5*A*B*C*COS(AL)*COS(GA)
      S(2,6)=-0.5*B*B*A*COS(GA)
      S(3,1)=-0.5*C*A*A*COS(BE)**2
      S(3,2)=-0.5*C*B*B*COS(AL)**2
      S(3,3)=-0.5*C*C*C
      S(3,4)=-0.5*C*C*B*COS(AL)
      S(3,5)=-0.5*C*C*A*COS(BE)
      S(3,6)=-0.5*A*B*C*COS(AL)*COS(BE)
      US=2.*COS(BE)*COS(GA)-COS(AL)*(COS(BE)**2+COS(GA)**2)
      S(4,1)=(A*A*US)/(2.*SIN(AL))
      S(4,2)=0.25*B*B*SIN(2.*AL)
      S(4,3)=0.25*C*C*SIN(2.*AL)
      S(4,4)=0.5*B*C*SIN(AL)
      S(4,5)=0.5*A*C*SIN(AL)*COS(GA)
      S(4,6)=0.5*A*B*SIN(AL)*COS(BE)
      S(5,1)=0.25*A*A*SIN(2.*BE)
      US=2.*COS(AL)*COS(GA)-COS(BE)*(COS(AL)**2+COS(GA)**2)
      S(5,2)=(B*B*US)/(2.*SIN(BE))
      S(5,3)=0.25*C*C*SIN(2.*BE)
      S(5,4)=0.5*B*C*SIN(BE)*COS(GA)
      S(5,5)=0.5*A*C*SIN(BE)
      S(5,6)=0.5*A*B*SIN(BE)*COS(AL)
      S(6,1)=0.25*A*A*SIN(2.*GA)
      S(6,2)=0.25*B*B*SIN(2.*GA)
      US=2.*COS(AL)*COS(BE)-COS(GA)*(COS(AL)**2+COS(BE)**2)
      S(6,3)=(C*C*US)/(2.*SIN(GA))
      S(6,4)=0.5*B*C*SIN(GA)*COS(BE)
      S(6,5)=0.5*A*C*SIN(GA)*COS(AL)
      S(6,6)=0.5*A*B*SIN(GA)
      RETURN
      END


      SUBROUTINE PW62(A,B,C,BE,S,ISYM)
C-----------------------------------------------------------------------
C     PARTIAL DERIVATIVES OF DIRECT LATTICE PARAMETERS(NOT TRICLINIC)
C-----------------------------------------------------------------------
      DIMENSION S(6,6)
      DO 10 I=1,6
      DO 10 J=1,6
   10 S(I,J)=0.0
      GO TO(70,60,50,30,20),ISYM
   20 S(1,1)=-0.5*A*A*A
      S(1,2)=0.0
      S(1,3)=-0.5*A*C*C*COS(BE)**2
      S(1,4)=-0.5*A*A*C*COS(BE)
      S(2,1)=0.0
      S(2,2)=-0.5*B*B*B
      S(2,3)=0.0
      S(2,4)=0.0
      S(3,1)=-0.5*A*A*C*COS(BE)**2
      S(3,2)=0.0
      S(3,3)=-0.5*C*C*C
      S(3,4)=-0.5*A*C*C*COS(BE)
      S(4,1)=0.25*A*A*SIN(2.*BE)
      S(4,2)=0.0
      S(4,3)=0.25*C*C*SIN(2.*BE)
      S(4,4)=0.5*A*C*SIN(BE)
      GO TO 80
   30 S(1,1)=-(3.*A*A*A)/8.
   40 S(1,2)=0.0
      S(2,1)=0.0
      S(2,2)=-0.5*C*C*C
      GO TO 80
   50 S(1,1)=-0.5*A*A*A
      S(1,2)=0.0
      S(1,3)=0.0
      S(2,1)=0.0
      S(2,2)=-0.5*B*B*B
      S(2,3)=0.0
      S(3,1)=0.0
      S(3,2)=0.0
      S(3,3)=-0.5*C*C*C
      GO TO 80
   60 S(1,1)=-0.5*A*A*A
      GO TO 40
   70 S(1,1)=-0.5*A*A*A
   80 RETURN
      END


      SUBROUTINE PW63(A11,A22,A33,A13,A12,A23,WAVE,ISYM,A,B,C,AL,BE,GA)
C-----------------------------------------------------------------------
C     CALC. OF DIRECT LATTICE PARAMETERS
C-----------------------------------------------------------------------
      A=0.0
      B=0.0
      C=0.0
      AL=0.0
      BE=0.0
      GA=0.0
      GO TO(10,20,30,40,50,90),ISYM
   10 A=WAVE/(2.*SQRT(A11))
      GO TO 190
   20 C=WAVE/(2.*SQRT(A22))
      GO TO 10
   30 B=WAVE/(2.*SQRT(A22))
      C=WAVE/(2.*SQRT(A33))
      GO TO 10
   40 A=WAVE/SQRT(3.*A11)
      C=WAVE/(2.*SQRT(A22))
      GO TO 190
   50 CBST=A13/(2.*SQRT(A11*A33))
      SBST=SQRT(1.-CBST*CBST)
      IF(CBST-0.0005) 60,70,70
   60 BE=1.570796
      GO TO 80
   70 TBST=SBST/CBST
      BST=ATAN(TBST)
      BE=3.141593-BST
   80 A=WAVE/(2.*SBST*SQRT(A11))
      B=WAVE/(2.*SQRT(A22))
      C=WAVE/(2.*SBST*SQRT(A33))
      GO TO 190
   90 S=4./(WAVE*WAVE)
      AST=SQRT(A11*S)
      BST=SQRT(A22*S)
      CST=SQRT(A33*S)
      CAST=A23/(2.*SQRT(A22*A33))
      CBST=A13/(2.*SQRT(A11*A33))
      CGST=A12/(2.*SQRT(A11*A22))
      VST=AST*BST*CST*SQRT(1.-CAST**2-CBST**2-CGST**2+2.*CAST*CBST*CGST)
      SAST=SQRT(1.-CAST**2)
      SBST=SQRT(1.-CBST**2)
      SGST=SQRT(1.-CGST**2)
      CAL=(CBST*CGST-CAST)/(SBST*SGST)
      CBE=(CAST*CGST-CBST)/(SAST*SGST)
      CGA=(CAST*CBST-CGST)/(SAST*SBST)
      IF(ABS(CAL)-0.0005) 100,110,110
  100 AL=1.570796
      GO TO 120
  110 SAL=SQRT(1.-CAL**2)
      AL=ATAN(SAL/CAL)
  120 IF(ABS(CBE)-0.0005) 130,140,140
  130 BE=1.570796
      GO TO 150
  140 SBE=SQRT(1.-CBE**2)
      BE=ATAN(SBE/CBE)
  150 IF(ABS(CGA)-0.0005) 160,170,170
  160 GA=1.570796
      GO TO 180
  170 SGA=SQRT(1.-CGA**2)
      GA=ATAN(SGA/CGA)
  180 A=(BST*CST*SAST)/VST
      B=(AST*CST*SBST)/VST
      C=(AST*BST*SGST)/VST
  190 RETURN
      END


      SUBROUTINE PW71(T2,WG,IHT,IKT,ILT,IQE,ISYM,N2,A11,A22,A33,A13,A12,
     >                A23,V2,V22,T22,NEKV)
C-----------------------------------------------------------------------
      DIMENSION T2(1),WG(1),IHT(1),IKT(1),ILT(1),IQE(1),T22(1)
      V2=0
      V22=0
      NEKV=0
      DO 90 I=1,N2
      IF(IQE(I)) 10,90,10
   10 GO TO(20,30,40,50,60,70),ISYM
   20 SQ=(IHT(I)**2+IKT(I)**2+ILT(I)**2)*A11
      GO TO 80
   30 SQ=(IHT(I)**2+IKT(I)**2)*A11+ILT(I)*ILT(I)*A22
      GO TO 80
   40 SQ=(IHT(I)**2)*A11+(IKT(I)**2)*A22+(ILT(I)**2)*A33
      GO TO 80
   50 SQ=(IHT(I)**2+IHT(I)*IKT(I)+IKT(I)**2)*A11+(ILT(I)**2)*A22
      GO TO 80
   60 SQ=(IHT(I)**2)*A11+(IKT(I)**2)*A22+(ILT(I)**2)*A33+
     >   (IHT(I)*ILT(I)*A13)
      GO TO 80
   70 SQ=(IHT(I)**2)*A11+(IKT(I)**2)*A22+(ILT(I)**2)*A33+
     >    IHT(I)*ILT(I)*A13+IHT(I)*IKT(I)*A12+IKT(I)*ILT(I)*A23
   80 V2=V2+WG(I)*((T2(I)-SQ))**2
      HA=SQRT(SQ)
      HB=SQRT(1.-SQ)
      TTH=114.59156*ATAN(HA/HB)
      V22=V22+(T22(I)-TTH)**2
      NEKV=NEKV+1
   90 CONTINUE
      RETURN
      END


      SUBROUTINE PW80(SSQT,T2,N2,IHT,IKT,ILT,IH,IK,IL,ISYM,INNE)
C-----------------------------------------------------------------------
      DIMENSION T2(1),IHT(1),IKT(1),ILT(1)
      I=1
   10 IF(I-N2) 30,30,20
   20 INNE=0
      RETURN
   30 IF(SSQT-T2(I)) 40,50,40
   40 I=I+1
      GO TO 10
   50 GO TO (60,80,100,120,130,140),ISYM
   60 IF((IH**2+IK**2+IL**2)-(IHT(I)**2+IKT(I)**2+ILT(I)**2)) 70,160,70
   70 INNE=0
      RETURN
   80 IF((IH**2+IK**2)-(IHT(I)**2+IKT(I)**2)) 70,90,70
   90 IF(IL**2-ILT(I)**2) 70,160,70
  100 IF(IH**2-IHT(I)**2) 70,110,70
  110 IF(IK**2-IKT(I)**2) 70,90,70
  120 IF((IH**2+IH*IK+IK**2)-(IHT(I)**2+IHT(I)*IKT(I)+ILT(I)**2))
     >  70,90,70
  130 IF(IH*IL-IHT(I)*ILT(I)) 70,100,70
  140 IF(IH*IK-IHT(I)*IKT(I)) 70,150,70
  150 IF(IK*IL-IKT(I)*ILT(I)) 70,130,70
  160 INNE=1
      RETURN
      END


      SUBROUTINE RDCEL(A11,A22,A33,A13,A12,A23,ISYM,WAVE)
C-----------------------------------------------------------------------
      GD=0.0174533
      SM=(WAVE*WAVE)/4.0
      GO TO(10,50,70,80,90,100),ISYM
   10 AST=1.0/A11
      A11=AST*AST*SM
      A22=0.0
   20 A33=0.0
   30 A13=0.0
   40 A12=0.0
      A23=0.0
      RETURN
   50 AST=1.0/A11
   60 A11=AST*AST*SM
      CST=1.0/A22
      A22=CST*CST*SM
      GO TO 20
   70 AST=1.0/A11
      A11=AST*AST*SM
      BST=1.0/A22
      A22=BST*BST*SM
      CST=1.0/A33
      A33=CST*CST*SM
      GO TO 30
   80 AST=2.0/(A11*SQRT(3.0))
      GO TO 60
   90 SB=SIN(GD*A13)
      CBST=-COS(GD*A13)
      AST=1.0/(A11*SB)
      BST=1.0/A22
      CST=1.0/(A33*SB)
      A11=AST*AST*SM
      A22=BST*BST*SM
      A33=CST*CST*SM
      A13=2.0*AST*CST*CBST*SM
      GO TO 40
  100 SA=SIN(A13*GD)
      CA=COS(A13*GD)
      SB=SIN(A12*GD)
      CB=COS(A12*GD)
      SC=SIN(A23*GD)
      CC=COS(A23*GD)
      TRIG=SQRT(1.0-CA*CA-CB*CB-CC*CC+2.0*CA*CB*CC)
      A=A11
      B=A22
      C=A33
      V=A*B*C*TRIG
      AST=(B*C*SA)/V
      BST=(A*C*SB)/V
      CST=(A*B*SC)/V
      A11=AST*AST*SM
      A22=BST*BST*SM
      A33=CST*CST*SM
      CAST=(CB*CC-CA)/(SB*SC)
      CBST=(CA*CC-CB)/(SA*SC)
      CCST=(CA*CB-CC)/(SA*SB)
      A13=2.0*AST*CST*CBST*SM
      A12=2.0*AST*BST*CCST*SM
      A23=2.0*BST*CST*CAST*SM
      RETURN
      END

c-----------------------------------------------------------------------

      subroutine ex (h,k,l,flag,numext,exsymb,ilatt)
c
c --- This is a routine that sets flag = 1 if the reflex is allowed,
c     else flag = 0
c
      integer h,k,l,flag
      logical extinct
      integer exsymb(50,5),numext,ilatt
c
      if (extinct(h,k,l,numext,exsymb,ilatt)) then
         flag = 0
      else
         flag = 1
      endif
c
      return
      end
    
c-----------------------------------------------------------------------

      subroutine upline(str80)
      character str80*80
      do 1 i = 1,80
         if (str80(i:i) .ge. 'a' .and. str80(i:i) .le. 'z') then
            str80(i:i) = char(ichar(str80(i:i))-ichar(' '))
         endif
    1 continue
      return
      end

c-----------------------------------------------------------------------

      subroutine strip(str80)
      character str80*80,temp*80
      temp = str80(istart(str80):80)
      str80 = temp
      return
      end

c-----------------------------------------------------------------------

      integer function nextbl(str80)
      character str80*80
      do 1 i=1,80
         if (str80(i:i) .eq. ' ') then
            goto 2
         endif
    1 continue
    2 continue
      nextbl = i
      return
      end

c-----------------------------------------------------------------------

      integer function lastch(str80)
      character str80*80
      do 1 i = 80,1,-1
         if (str80(i:i) .ne. ' ') then
            lastch = i
            return
         endif
    1 continue
      lastch = 0
      return
      end

c-----------------------------------------------------------------------

      logical function even(i)
      if (mod(i,2) .eq. 0) then
         even = .true.
      else
         even = .false.
      endif
      return
      end

c-----------------------------------------------------------------------

      subroutine get_real (str,istat,temp)
      character str*80
      read (str,'(f80.0)',iostat=istat) temp
      return
      end

c-----------------------------------------------------------------------

      subroutine get_int (str,istat,itemp)
      character str*80,s*80
      call strip(str)
      n = lastch(str)
      s = ' '
      s(80-n+1:80) = str(1:n)
      read (s,'(I80)',iostat=istat) itemp
      return
      end
c-----------------------------------------------------------------------
      subroutine wait(lukeys,ludisp)
      write(LUDISP,'(A,$)') ' Press RETURN to continue:'
      read(lukeys,*)
      return
      end
c-----------------------------------------------------------------------

      subroutine pw58(ihi,iki,ili,n1,isym)
      dimension ihi(1),iki(1),ili(1)
      do 99 i=1,n1
      ih=ihi(i)
      ik=iki(i)
      il=ili(i)
      go to(11,12,99,12,15,16),isym
   11 if (ih .ge. ik .and. ik .ge. il) then
        goto 30
       elseif (ih .ge. ik .and. ik .lt. il) then
        ia=ik
        ik=il
        il=ia
        goto 11
       elseif (ih .lt. ik) then
        ia=ik
        ik=ih
        ih=ia
        goto 11
       else
       goto 30
      endif
   12 if (ih .ge. ik) then
       goto 30
       else
        ia=ih
        ih=ik
        ik=ia
       endif
      goto 30
   15 if(il .lt. 0) then
       ih=-ih
       il=-il
       endif
       goto 30
   16 if(il .lt. 0) then
       ih=-ih
       ik=-ik
       il=-il
      endif    
   30 ihi(i)=ih
      iki(i)=ik
      ili(i)=il
   99 continue
      return
      end